@@ -45,13 +45,13 @@ import numpy as np
 
 
 def correlation_matrix(basis, rho=None):
-    """
-    Given a basis set of operators :math:`\\{a\\}_n`, calculate the correlation
+    r"""
+    Given a basis set of operators :math:`\{a\}_n`, calculate the correlation
     matrix:
 
     .. math::
 
-        C_{mn} = \\langle a_m a_n \\rangle
+        C_{mn} = \langle a_m a_n \rangle
 
     Parameters
     ----------
@@ -82,21 +82,21 @@ def correlation_matrix(basis, rho=None):
 
 
 def covariance_matrix(basis, rho, symmetrized=True):
-    """
+    r"""
     Given a basis set of operators :math:`\{a\}_n`, calculate the covariance
     matrix:
 
     .. math::
 
-        V_{mn} = \\frac{1}{2}\\langle a_m a_n + a_n a_m \\rangle -
-        \\langle a_m \\rangle \\langle a_n\\rangle
+        V_{mn} = \frac{1}{2}\langle a_m a_n + a_n a_m \rangle -
+        \langle a_m \rangle \langle a_n\rangle
 
     or, if of the optional argument `symmetrized=False`,
 
     .. math::
 
-        V_{mn} = \\langle a_m a_n\\rangle -
-        \\langle a_m \\rangle \\langle a_n\\rangle
+        V_{mn} = \langle a_m a_n\rangle -
+        \langle a_m \rangle \langle a_n\rangle
 
     Parameters
     ----------
@@ -187,12 +187,12 @@ def correlation_matrix_quadrature(a1, a2, rho=None, g=np.sqrt(2)):
 
 
 def wigner_covariance_matrix(a1=None, a2=None, R=None, rho=None, g=np.sqrt(2)):
-    """
+    r"""
     Calculates the Wigner covariance matrix
-    :math:`V_{ij} = \\frac{1}{2}(R_{ij} + R_{ji})`, given
+    :math:`V_{ij} = \frac{1}{2}(R_{ij} + R_{ji})`, given
     the quadrature correlation matrix
-    :math:`R_{ij} = \\langle R_{i} R_{j}\\rangle -
-    \\langle R_{i}\\rangle \\langle R_{j}\\rangle`, where
+    :math:`R_{ij} = \langle R_{i} R_{j}\rangle -
+    \langle R_{i}\rangle \langle R_{j}\rangle`, where
     :math:`R = (q_1, p_1, q_2, p_2)^T` is the vector with quadrature operators
     for the two modes.
 
