@@ -56,14 +56,13 @@ Note the methods in these classes were inspired by:
 DYNAMO - Dynamic Framework for Quantum Optimal Control
 See Machnes et.al., arXiv.1011.4874
 """
-import os
 import warnings
 import numpy as np
 import scipy.linalg as la
 import scipy.sparse as sp
 # QuTiP
 from qutip.qobj import Qobj
-from qutip.sparse import sp_eigs, _dense_eigs, eigh
+from qutip.sparse import sp_eigs, eigh
 import qutip.settings as settings
 # QuTiP logging
 import qutip.logging_utils as logging
@@ -515,13 +514,15 @@ class Dynamics(object):
         """
         The level of data dumping that will occur during the time evolution
         calculation.
-         - NONE : No processing data dumped (Default)
-         - SUMMARY : A summary of each time evolution will be recorded
-         - FULL : All operators used or created in the calculation dumped
-         - CUSTOM : Some customised level of dumping
+
+        - NONE : No processing data dumped (Default)
+        - SUMMARY : A summary of each time evolution will be recorded
+        - FULL : All operators used or created in the calculation dumped
+        - CUSTOM : Some customised level of dumping
+
         When first set to CUSTOM this is equivalent to SUMMARY. It is then up
-        to the user to specify which operators are dumped
-        WARNING: FULL could consume a lot of memory!
+        to the user to specify which operators are dumped.  WARNING: FULL could
+        consume a lot of memory!
         """
         if self.dump is None:
             lvl = 'NONE'
@@ -868,11 +869,13 @@ class Dynamics(object):
         """
         phase_application : scalar(string), default='preop'
         Determines how the phase is applied to the dynamics generators
-         - 'preop'  : P = expm(phase*dyn_gen)
-         - 'postop' : P = expm(dyn_gen*phase)
-         - 'custom' : Customised phase application
+
+        - 'preop'  : P = expm(phase*dyn_gen)
+        - 'postop' : P = expm(dyn_gen*phase)
+        - 'custom' : Customised phase application
+
         The 'custom' option assumes that the _apply_phase method has been
-        set to a custom function
+        set to a custom function.
         """
         return self._phase_application
 
@@ -1794,7 +1797,7 @@ class DynamicsSymplectic(Dynamics):
 
     @property
     def dyn_gen_phase(self):
-        """
+        r"""
         The phasing operator for the symplectic group generators
         usually refered to as \Omega
         By default this is applied as 'postop' dyn_gen*-\Omega
