@@ -37,12 +37,18 @@ import numpy as np
 import scipy.sparse as sp
 from qutip.qobj import Qobj
 
+import warnings
+
 
 class eseries():
     """
     Class representation of an exponential-series expansion of
     time-dependent quantum objects.
 
+    .. deprecated:: 4.6.0
+        :obj:`~eseries` will be removed in QuTiP 5.  Please use :obj:`~QobjEvo`
+        for general time-dependence.
+
     Attributes
     ----------
     ampl : ndarray
@@ -67,6 +73,11 @@ class eseries():
     __array_priority__ = 101
 
     def __init__(self, q=None, s=np.array([])):
+        warnings.warn(
+            "eseries is to be removed in QuTiP 5.0,"
+            " consider swapping to QobjEvo for general time dependence.",
+            DeprecationWarning, stacklevel=2,
+        )
 
         if isinstance(s, (int, float, complex)):
             s = np.array([s])
