@@ -44,11 +44,31 @@ from qutip.superoperator import liouvillian, mat2vec, vec2mat
 from qutip.solver import Result
 from qutip.operators import qzero
 
+# Only used for deprecation warnings.
+import functools
+import warnings
+
+
+def _deprecate(alternative):
+    def decorated(f):
+        message = (
+            f"{f.__name__} is to be removed in QuTiP 5.0"
+            f", consider swapping to {alternative}."
+        )
+
+        @functools.wraps(f)
+        def out(*args, **kwargs):
+            warnings.warn(message, DeprecationWarning, stacklevel=2)
+            return f(*args, **kwargs)
+        return out
+    return decorated
+
 
 # -----------------------------------------------------------------------------
 # pass on to wavefunction solver or master equation solver depending on whether
 # any collapse operators were given.
 #
+@_deprecate("mesolve")
 def essolve(H, rho0, tlist, c_op_list, e_ops):
     """
     Evolution of a state vector or density matrix (`rho0`) for a given
@@ -57,6 +77,12 @@ def essolve(H, rho0, tlist, c_op_list, e_ops):
     the state vector at arbitrary points in time (`tlist`), or the
     expectation values of the supplied operators (`e_ops`).
 
+    .. deprecated:: 4.6.0
+        :obj:`~essolev` will be removed in QuTiP 5.  Please use :obj:`~sesolve`
+        or :obj:`~mesolve` for general-purpose integration of the
+        Schroedinger/Lindblad master equation.  This will likely be faster than
+        :obj:`~essolve` for you.
+
     Parameters
     ----------
     H : qobj/function_type
@@ -118,11 +144,17 @@ def essolve(H, rho0, tlist, c_op_list, e_ops):
 # -----------------------------------------------------------------------------
 #
 #
+@_deprecate("direct eigenstate and -value calculation")
 def ode2es(L, rho0):
     """Creates an exponential series that describes the time evolution for the
     initial density matrix (or state vector) `rho0`, given the Liouvillian
     (or Hamiltonian) `L`.
 
+    .. deprecated:: 4.6.0
+        :obj:`~ode2es` will be removed in QuTiP 5.  Please use
+        :obj:`Qobj.eigenstates` to get the eigenstates and -values, and use
+        :obj:`~QobjEvo` for general time-dependence.
+
     Parameters
     ----------
     L : qobj
@@ -189,7 +221,7 @@ def ode2es(L, rho0):
 
         out = None
         for i in range(rlen):
-            qo = Qobj(np.matrix(vv[:, i]).T, dims=rho0.dims, shape=rho0.shape)
+            qo = Qobj(np.array(vv[:, i]).T, dims=rho0.dims, shape=rho0.shape)
             if out:
                 out += eseries(qo, -1.0j * w[i])
             else:
