@@ -33,16 +33,33 @@
 """
 This module contains utility functions for using QuTiP with IPython notebooks.
 """
-
-__all__ = ['version_table', 'parfor', 'plot_animation', 'parallel_map']
-
 from qutip.ui.progressbar import BaseProgressBar
+from qutip.utilities import _blas_info
+import IPython
+
+#IPython parallel routines moved to ipyparallel in V4
+#IPython parallel routines not in Anaconda by default
+if IPython.version_info[0] >= 4:
+    try:
+        from ipyparallel import Client
+        __all__ = ['version_table', 'parfor', 'plot_animation',
+                    'parallel_map', 'HTMLProgressBar']
+    except:
+         __all__ = ['version_table', 'plot_animation', 'HTMLProgressBar']
+else:
+    try:
+        from IPython.parallel import Client
+        __all__ = ['version_table', 'parfor', 'plot_animation',
+                    'parallel_map', 'HTMLProgressBar']
+    except:
+         __all__ = ['version_table', 'plot_animation', 'HTMLProgressBar']
+
 
-from IPython.parallel import Client
 from IPython.display import HTML, Javascript, display
 
 import matplotlib.pyplot as plt
 from matplotlib import animation
+from base64 import b64encode
 
 import datetime
 import uuid
@@ -78,18 +95,20 @@ def version_table(verbose=False):
     html = "<table>"
     html += "<tr><th>Software</th><th>Version</th></tr>"
 
-    packages = {"QuTiP": qutip.__version__,
-                "Numpy": numpy.__version__,
-                "SciPy": scipy.__version__,
-                "matplotlib": matplotlib.__version__,
-                "Cython": Cython.__version__,
-                "Python": sys.version,
-                "IPython": IPython.__version__,
-                "OS": "%s [%s]" % (os.name, sys.platform)
-                }
+    packages = [("QuTiP", qutip.__version__),
+                ("Numpy", numpy.__version__),
+                ("SciPy", scipy.__version__),
+                ("matplotlib", matplotlib.__version__),
+                ("Cython", Cython.__version__),
+                ("Number of CPUs", qutip.hardware_info.hardware_info()['cpus']),
+                ("BLAS Info", _blas_info()),
+                ("IPython", IPython.__version__),
+                ("Python", sys.version),
+                ("OS", "%s [%s]" % (os.name, sys.platform))
+                ]
 
-    for name in packages:
-        html += "<tr><td>%s</td><td>%s</td></tr>" % (name, packages[name])
+    for name, version in packages:
+        html += "<tr><td>%s</td><td>%s</td></tr>" % (name, version)
 
     if verbose:
         html += "<tr><th colspan='2'>Additional information</th></tr>"
@@ -354,7 +373,7 @@ def parallel_map(task, values, task_args=None, task_kwargs=None,
 
 
 def plot_animation(plot_setup_func, plot_func, result, name="movie",
-                   verbose=False):
+                   writer="avconv", codec="libx264", verbose=False):
     """
     Create an animated plot of a Result object, as returned by one of
     the qutip evolution solvers.
@@ -365,12 +384,12 @@ def plot_animation(plot_setup_func, plot_func, result, name="movie",
     fig, axes = plot_setup_func(result)
 
     def update(n):
-        plot_func(result, n, fig=fig, axes=axes)
+        return plot_func(result, n, fig=fig, axes=axes)
 
     anim = animation.FuncAnimation(
         fig, update, frames=len(result.times), blit=True)
 
-    anim.save(name + '.mp4', fps=10, writer="avconv", codec="libx264")
+    anim.save(name + '.mp4', fps=10, writer=writer, codec=codec)
 
     plt.close(fig)
 
@@ -378,7 +397,7 @@ def plot_animation(plot_setup_func, plot_func, result, name="movie",
         print("Created %s.m4v" % name)
 
     video = open(name + '.mp4', "rb").read()
-    video_encoded = video.encode("base64")
+    video_encoded = b64encode(video).decode("ascii")
     video_tag = '<video controls src="data:video/x-m4v;base64,{0}">'.format(
         video_encoded)
     return HTML(video_tag)
