@@ -48,7 +48,7 @@ from qutip.states import ket2dm
 from qutip.superop_reps import to_kraus, to_stinespring, to_choi, _super_to_superpauli, to_super
 from qutip.superoperator import operator_to_vector, vector_to_operator
 from qutip.operators import qeye
-from qutip.semidefinite import dnorm_problem
+from qutip.semidefinite import dnorm_problem, dnorm_sparse_problem
 import qutip.settings as settings
 
 import qutip.logging_utils as logging
@@ -56,7 +56,7 @@ logger = logging.get_logger()
 
 try:
     import cvxpy
-except:
+except ImportError:
     cvxpy = None
 
 
@@ -81,9 +81,7 @@ def fidelity(A, B):
     --------
     >>> x = fock_dm(5,3)
     >>> y = coherent_dm(5,1)
-    >>> fidelity(x,y)
-    0.24104350624628332
-
+    >>> np.testing.assert_almost_equal(fidelity(x,y), 0.24104350624628332)
     """
     if A.isket or A.isbra:
         # Take advantage of the fact that the density operator for A
@@ -182,9 +180,7 @@ def tracedist(A, B, sparse=False, tol=0):
     --------
     >>> x=fock_dm(5,3)
     >>> y=coherent_dm(5,1)
-    >>> tracedist(x,y)
-    0.9705143161472971
-
+    >>> np.testing.assert_almost_equal(tracedist(x,y), 0.9705143161472971)
     """
     if A.isket or A.isbra:
         A = ket2dm(A)
@@ -323,13 +319,8 @@ def hellinger_dist(A, B, sparse=False, tol=0):
     --------
     >>> x=fock_dm(5,3)
     >>> y=coherent_dm(5,1)
-    >>> hellinger_dist(x,y)
-    1.3725145002591095
-
+    >>> np.testing.assert_almost_equal(hellinger_dist(x,y), 1.3725145002591095)
     """
-    if A.dims != B.dims:
-        raise TypeError("A and B do not have same dimensions.")
-
     if A.isket or A.isbra:
         sqrtmA = ket2dm(A)
     else:
@@ -339,17 +330,20 @@ def hellinger_dist(A, B, sparse=False, tol=0):
     else:
         sqrtmB = B.sqrtm(sparse=sparse, tol=tol)
 
-    product = sqrtmA*sqrtmB
+    if sqrtmA.dims != sqrtmB.dims:
+        raise TypeError("A and B do not have compatible dimensions.")
 
+    product = sqrtmA*sqrtmB
     eigs = sp_eigs(product.data,
                    isherm=product.isherm, vecs=False, sparse=sparse, tol=tol)
-    #np.maximum() is to avoid nan appearing sometimes due to numerical
-    #instabilities causing np.sum(eigs) slightly (~1e-8) larger than 1
-    #when hellinger_dist(A, B) is called for A=B
+    # np.maximum() is to avoid nan appearing sometimes due to numerical
+    # instabilities causing np.sum(eigs) slightly (~1e-8) larger than 1
+    # when hellinger_dist(A, B) is called for A=B
     return np.sqrt(2.0 * np.maximum(0., (1.0 - np.real(np.sum(eigs)))))
 
 
-def dnorm(A, B=None, solver="CVXOPT", verbose=False, force_solve=False):
+def dnorm(A, B=None, solver="CVXOPT", verbose=False, force_solve=False,
+          sparse=True):
     """
     Calculates the diamond norm of the quantum map q_oper, using
     the simplified semidefinite program of [Wat12]_.
@@ -361,18 +355,18 @@ def dnorm(A, B=None, solver="CVXOPT", verbose=False, force_solve=False):
     A : Qobj
         Quantum map to take the diamond norm of.
     B : Qobj or None
-        If provided, the diamond norm of :math:`A - B` is
-        taken instead.
+        If provided, the diamond norm of :math:`A - B` is taken instead.
     solver : str
-        Solver to use with CVXPY. One of "CVXOPT" (default)
-        or "SCS". The latter tends to be significantly faster,
-        but somewhat less accurate.
+        Solver to use with CVXPY. One of "CVXOPT" (default) or "SCS". The
+        latter tends to be significantly faster, but somewhat less accurate.
     verbose : bool
-        If True, prints additional information about the
-        solution.
+        If True, prints additional information about the solution.
     force_solve : bool
         If True, forces dnorm to solve the associated SDP, even if a special
         case is known for the argument.
+    sparse : bool
+        Whether to use sparse matrices in the convex optimisation problem.
+        Default True.
 
     Returns
     -------
@@ -438,7 +432,7 @@ def dnorm(A, B=None, solver="CVXOPT", verbose=False, force_solve=False):
 
     # Force the input superoperator to be a Choi matrix.
     J = to_choi(A)
-    
+
     if B is not None:
         J -= to_choi(B)
 
@@ -447,35 +441,43 @@ def dnorm(A, B=None, solver="CVXOPT", verbose=False, force_solve=False):
     # of the dual map of Lambda. We can evaluate that norm much more
     # easily if Lambda is completely positive, since then the largest
     # eigenvalue is the same as the largest singular value.
-    
+
     if not force_solve and J.iscp:
         S_dual = to_super(J.dual_chan())
         vec_eye = operator_to_vector(qeye(S_dual.dims[1][1]))
         op = vector_to_operator(S_dual * vec_eye)
         # The 2-norm was not implemented for sparse matrices as of the time
         # of this writing. Thus, we must yet again go dense.
-        return la.norm(op.data.todense(), 2)
-    
+        return la.norm(op.full(), 2)
+
     # If we're still here, we need to actually solve the problem.
 
     # Assume square...
     dim = np.prod(J.dims[0][0])
-    
-    # The constraints only depend on the dimension, so
-    # we can cache them efficiently.
-    problem, Jr, Ji, X, rho0, rho1 = dnorm_problem(dim)
-    
-    # Load the parameters with the Choi matrix passed in.
+
     J_dat = J.data
-    
-    Jr.value = sp.csr_matrix((J_dat.data.real, J_dat.indices, J_dat.indptr), 
-                             shape=J_dat.shape)
-   
-    Ji.value = sp.csr_matrix((J_dat.data.imag, J_dat.indices, J_dat.indptr),
-                             shape=J_dat.shape)
-    # Finally, set up and run the problem.
+
+    if not sparse:
+        # The parameters and constraints only depend on the dimension, so
+        # we can cache them efficiently.
+        problem, Jr, Ji = dnorm_problem(dim)
+
+        # Load the parameters with the Choi matrix passed in.
+        Jr.value = sp.csr_matrix((J_dat.data.real, J_dat.indices,
+                                  J_dat.indptr),
+                                 shape=J_dat.shape).toarray()
+
+        Ji.value = sp.csr_matrix((J_dat.data.imag, J_dat.indices,
+                                  J_dat.indptr),
+                                 shape=J_dat.shape).toarray()
+    else:
+
+        # The parameters do not depend solely on the dimension,
+        # so we can not cache them efficiently.
+        problem = dnorm_sparse_problem(dim, J_dat)
+
     problem.solve(solver=solver, verbose=verbose)
-    
+
     return problem.value
 
 
