@@ -44,7 +44,6 @@ __all__ = ['jmat', 'spin_Jx', 'spin_Jy', 'spin_Jz', 'spin_Jm', 'spin_Jp',
 
 import numbers
 import numpy as np
-import scipy
 import scipy.sparse as sp
 from qutip.qobj import Qobj
 from qutip.fastsparse import fast_csr_matrix, fast_identity
@@ -73,7 +72,7 @@ def jmat(j, *args):
 
     Examples
     --------
-    >>> jmat(1)
+    >>> jmat(1) # doctest: +SKIP
     [ Quantum object: dims = [[3], [3]], \
 shape = [3, 3], type = oper, isHerm = True
     Qobj data =
@@ -268,7 +267,7 @@ def sigmap():
 
     Examples
     --------
-    >>> sigmap()
+    >>> sigmap() # doctest: +SKIP
     Quantum object: dims = [[2], [2]], \
 shape = [2, 2], type = oper, isHerm = False
     Qobj data =
@@ -284,7 +283,7 @@ def sigmam():
 
     Examples
     --------
-    >>> sigmam()
+    >>> sigmam() # doctest: +SKIP
     Quantum object: dims = [[2], [2]], \
 shape = [2, 2], type = oper, isHerm = False
     Qobj data =
@@ -300,7 +299,7 @@ def sigmax():
 
     Examples
     --------
-    >>> sigmax()
+    >>> sigmax() # doctest: +SKIP
     Quantum object: dims = [[2], [2]], \
 shape = [2, 2], type = oper, isHerm = False
     Qobj data =
@@ -308,7 +307,7 @@ shape = [2, 2], type = oper, isHerm = False
      [ 1.  0.]]
 
     """
-    return 2.0 * jmat(1.0 / 2, 'x')
+    return 2 * jmat(1 / 2, 'x')
 
 
 def sigmay():
@@ -316,7 +315,7 @@ def sigmay():
 
     Examples
     --------
-    >>> sigmay()
+    >>> sigmay() # doctest: +SKIP
     Quantum object: dims = [[2], [2]], \
 shape = [2, 2], type = oper, isHerm = True
     Qobj data =
@@ -324,7 +323,7 @@ shape = [2, 2], type = oper, isHerm = True
      [ 0.+1.j  0.+0.j]]
 
     """
-    return 2.0 * jmat(1.0 / 2, 'y')
+    return 2 * jmat(1 / 2, 'y')
 
 
 def sigmaz():
@@ -332,7 +331,7 @@ def sigmaz():
 
     Examples
     --------
-    >>> sigmaz()
+    >>> sigmaz() # doctest: +SKIP
     Quantum object: dims = [[2], [2]], \
 shape = [2, 2], type = oper, isHerm = True
     Qobj data =
@@ -340,7 +339,7 @@ shape = [2, 2], type = oper, isHerm = True
      [ 0. -1.]]
 
     """
-    return 2.0 * jmat(1.0 / 2, 'z')
+    return 2 * jmat(1 / 2, 'z')
 
 
 #
@@ -366,7 +365,7 @@ def destroy(N, offset=0):
 
     Examples
     --------
-    >>> destroy(4)
+    >>> destroy(4) # doctest: +SKIP
     Quantum object: dims = [[4], [4]], \
 shape = [4, 4], type = oper, isHerm = False
     Qobj data =
@@ -408,7 +407,7 @@ def create(N, offset=0):
 
     Examples
     --------
-    >>> create(4)
+    >>> create(4) # doctest: +SKIP
     Quantum object: dims = [[4], [4]], \
 shape = [4, 4], type = oper, isHerm = False
     Qobj data =
@@ -500,14 +499,14 @@ def qeye(dimensions):
 
     Examples
     --------
-    >>> qeye(3)
+    >>> qeye(3) # doctest: +SKIP
     Quantum object: dims = [[3], [3]], shape = (3, 3), type = oper, \
 isherm = True
     Qobj data =
     [[ 1.  0.  0.]
      [ 0.  1.  0.]
      [ 0.  0.  1.]]
-    >>> qeye([2,2])
+    >>> qeye([2,2]) # doctest: +SKIP
     Quantum object: dims = [[2, 2], [2, 2]], shape = (4, 4), type = oper, \
 isherm = True
     Qobj data =
@@ -604,7 +603,7 @@ def num(N, offset=0):
 
     Examples
     --------
-    >>> num(4)
+    >>> num(4) # doctest: +SKIP
     Quantum object: dims = [[4], [4]], \
 shape = [4, 4], type = oper, isHerm = True
     Qobj data =
@@ -652,7 +651,7 @@ def squeeze(N, z, offset=0):
 
     Examples
     --------
-    >>> squeeze(4, 0.25)
+    >>> squeeze(4, 0.25) # doctest: +SKIP
     Quantum object: dims = [[4], [4]], \
 shape = [4, 4], type = oper, isHerm = False
     Qobj data =
@@ -718,7 +717,7 @@ def displace(N, alpha, offset=0):
 
     Examples
     ---------
-    >>> displace(4,0.25)
+    >>> displace(4,0.25) # doctest: +SKIP
     Quantum object: dims = [[4], [4]], \
 shape = [4, 4], type = oper, isHerm = False
     Qobj data =
@@ -801,7 +800,7 @@ def qdiags(diagonals, offsets, dims=None, shape=None):
 
     Examples
     --------
-    >>> qdiags(sqrt(range(1, 4)), 1)
+    >>> qdiags(sqrt(range(1, 4)), 1) # doctest: +SKIP
     Quantum object: dims = [[4], [4]], \
 shape = [4, 4], type = oper, isherm = False
     Qobj data =
@@ -976,7 +975,7 @@ def charge(Nmax, Nmin=None, frac = 1):
 def tunneling(N, m=1):
     """
     Tunneling operator with elements of the form
-    :math:`\sum |N><N+m| + |N+m><N|`.
+    :math:`\\sum |N><N+m| + |N+m><N|`.
 
     Parameters
     ----------
