@@ -43,10 +43,11 @@ from qutip.operators import identity
 from qutip.qip.operations.gates import expand_operator, globalphase
 from qutip.tensor import tensor
 from qutip.mesolve import mesolve
+from qutip.mcsolve import mcsolve
 from qutip.qip.circuit import QubitCircuit
 from qutip.qip.noise import (
     Noise, RelaxationNoise, DecoherenceNoise,
-    ControlAmpNoise, RandomNoise, UserNoise, process_noise)
+    ControlAmpNoise, RandomNoise, process_noise)
 from qutip.qip.pulse import Pulse, Drift, _merge_qobjevo, _fill_coeff
 
 
@@ -56,12 +57,10 @@ __all__ = ['Processor']
 class Processor(object):
     """
     A simulator of a quantum device based on the QuTiP solver
-    :func:`qutip.mesolve`.
-    It is defined by the available driving Hamiltonian and
-    the decoherence time for each component systems.
-    The processor can simulate the evolution under the given
-    control pulses. Noisy evolution is supported by
-    :class:`qutip.qip.Noise` and can be added to the processor.
+    :func:`qutip.mesolve`.  It is defined by the available driving Hamiltonian
+    and the decoherence time for each component systems.  The processor can
+    simulate the evolution under the given control pulses. Noisy evolution is
+    supported by :class:`.Noise` and can be added to the processor.
 
     Parameters
     ----------
@@ -83,24 +82,24 @@ class Processor(object):
 
     spline_kind: str, optional
         Type of the coefficient interpolation. Default is "step_func"
-        Note that they have different requirement for the length of `coeff'.
+        Note that they have different requirement for the length of ``coeff``.
 
-        -"step_func":
-        The coefficient will be treated as a step function.
-        E.g. ``tlist=[0,1,2]`` and ``coeff=[3,2]``, means that the coefficient
-        is 3 in t=[0,1) and 2 in t=[2,3). It requires
-        ``len(coeff)=len(tlist)-1`` or ``len(coeff)=len(tlist)``, but
-        in the second case the last element of `coeff` has no effect.
+        - "step_func":
+          The coefficient will be treated as a step function.  E.g.
+          ``tlist=[0,1,2]`` and ``coeff=[3,2]``, means that the coefficient is
+          3 in t=[0,1) and 2 in t=[2,3). It requires
+          ``len(coeff)=len(tlist)-1`` or ``len(coeff)=len(tlist)``, but in the
+          second case the last element of `coeff` has no effect.
 
-        -"cubic": Use cubic interpolation for the coefficient. It requires
-        ``len(coeff)=len(tlist)``
+        - "cubic": Use cubic interpolation for the coefficient. It requires
+          ``len(coeff)=len(tlist)``
 
     Attributes
     ----------
     N: int
         The number of component systems.
 
-    pulses: list of :class:`qutip.qip.Pulse`
+    pulses: list of :class:`.Pulse`
         A list of control pulses of this device
 
     t1: float or list
@@ -111,11 +110,11 @@ class Processor(object):
         Characterize the decoherence of dephasing for
         each qubit.
 
-    noise: :class:`qutip.qip.Noise`, optional
+    noise: :class:`.Noise`, optional
         A list of noise objects. They will be processed when creating the
         noisy :class:`qutip.QobjEvo` from the processor or run the simulation.
 
-    drift: :class:`qutip.qip.Drift`
+    drift: :class:`qutip.qip.pulse.Drift`
         A `Drift` object representing the drift Hamiltonians.
 
     dims: list
@@ -125,7 +124,7 @@ class Processor(object):
 
     spline_kind: str
         Type of the coefficient interpolation.
-        See parameters of :class:`qutip.qip.Processor` for details.
+        See parameters of :class:`.Processor` for details.
     """
     def __init__(self, N, t1=None, t2=None,
                  dims=None, spline_kind="step_func"):
@@ -139,11 +138,21 @@ class Processor(object):
             self.dims = [2] * N
         else:
             self.dims = dims
+        self.pulse_mode = "discrete"
         self.spline_kind = spline_kind
 
+    @property
+    def num_qubits(self):
+        return self.N
+
+    @num_qubits.setter
+    def num_qubits(self, value):
+        self.N = value
+
     def add_drift(self, qobj, targets, cyclic_permutation=False):
         """
-        Add one Hamiltonian to the drift Hamiltonians
+        Add a drift Hamiltonians. The drift Hamiltonians are intrinsic
+        of the quantum system and cannot be controlled by external field.
 
         Parameters
         ----------
@@ -174,7 +183,7 @@ class Processor(object):
                     label=None):
         """
         Add a control Hamiltonian to the processor. It creates a new
-        :class:`qutip.qip.Pulse`
+        :class:`.Pulse`
         object for the device that is turned off
         (``tlist = None``, ``coeff = None``). To activate the pulse, one
         can set its `tlist` and `coeff`.
@@ -217,12 +226,30 @@ class Processor(object):
                           label=temp_label))
         else:
             self.pulses.append(
-                Pulse(qobj, targets, spline_kind=self.spline_kind, label=label))
+                Pulse(qobj, targets, spline_kind=self.spline_kind, label=label)
+                )
+
+    def find_pulse(self, pulse_name):
+        if isinstance(pulse_name, str):
+            try:
+                return self.pulses[self.pulse_dict[pulse_name]]
+            except (KeyError):
+                raise KeyError(
+                    "Pulse name {} undefined. "
+                    "Please define it in the attribute "
+                    "`pulse_dict`.".format(pulse_name))
+        elif isinstance(pulse_name, int):
+            return self.pulses[pulse_name]
+        else:
+            raise TypeError(
+                "pulse_name is either a string or an integer, not "
+                "{}".format(type(pulse_name))
+                )
 
     @property
     def ctrls(self):
         """
-        list: A list of Hamiltonian of all pulses.
+        A list of Hamiltonians of all pulses.
         """
         result = []
         for pulse in self.pulses:
@@ -241,30 +268,55 @@ class Processor(object):
 
     @coeffs.setter
     def coeffs(self, coeffs_list):
-        if len(coeffs_list) != len(self.pulses):
-            raise ValueError("The row number of coeffs must be same "
-                             "as the number of control pulses.")
         for i, coeff in enumerate(coeffs_list):
             self.pulses[i].coeff = coeff
 
-    def get_full_tlist(self):
+    @property
+    def pulse_mode(self):
+        if self.spline_kind == "step_func":
+            return "discrete"
+        elif self.spline_kind == "cubic":
+            return "continuous"
+        else:
+            raise ValueError(
+                "Saved spline_kind not understood.")
+
+    @pulse_mode.setter
+    def pulse_mode(self, mode):
+        if mode == "discrete":
+            spline_kind = "step_func"
+        elif mode == "continuous":
+            spline_kind = "cubic"
+        else:
+            raise ValueError(
+                "Pulse mode must be either discrete or continuous.")
+
+        self.spline_kind = spline_kind
+        for pulse in self.pulses:
+            pulse.spline_kind = spline_kind
+
+    def get_full_tlist(self, tol=1.0e-10):
         """
         Return the full tlist of the ideal pulses.
-        It means that if different `tlist`s are present, they will be merged
-        to one with all time points stored in a sorted array.
+        If different pulses have different time steps,
+        it will collect all the time steps in a sorted array.
 
         Returns
         -------
         full_tlist: array-like 1d
             The full time sequence for the ideal evolution.
         """
-        all_tlists = [pulse.tlist
+        full_tlist = [pulse.tlist
                       for pulse in self.pulses if pulse.tlist is not None]
-        if not all_tlists:
+        if not full_tlist:
             return None
-        return np.unique(np.sort(np.hstack(all_tlists)))
+        full_tlist = np.unique(np.sort(np.hstack(full_tlist)))
+        # account for inaccuracy in float-point number
+        full_tlist = np.concatenate(
+            (full_tlist[:1], full_tlist[1:][np.diff(full_tlist) > tol]))
+        return full_tlist
 
-    def get_full_coeffs(self):
+    def get_full_coeffs(self, full_tlist=None):
         """
         Return the full coefficients in a 2d matrix form.
         Each row corresponds to one pulse. If the `tlist` are
@@ -282,18 +334,23 @@ class Processor(object):
         self._is_pulses_valid()
         if not self.pulses:
             return np.array((0, 0), dtype=float)
-        full_tlist = self.get_full_tlist()
+        if full_tlist is None:
+            full_tlist = self.get_full_tlist()
         coeffs_list = []
         for pulse in self.pulses:
-            if isinstance(pulse.coeff, bool):
-                if pulse.coeff:
-                    coeffs_list.append(np.ones(full_tlist))
-                else:
-                    coeffs_list.append(np.zeros(full_tlist))
-            if not isinstance(pulse.coeff, np.ndarray):
+            if pulse.tlist is None and pulse.coeff is None:
+                coeffs_list.append(np.zeros(len(full_tlist)))
+                continue
+            if not isinstance(pulse.coeff, (bool, np.ndarray)):
                 raise ValueError(
                     "get_full_coeffs only works for "
                     "NumPy array or bool coeff.")
+            if isinstance(pulse.coeff, bool):
+                if pulse.coeff:
+                    coeffs_list.append(np.ones(len(full_tlist)))
+                else:
+                    coeffs_list.append(np.zeros(len(full_tlist)))
+                continue
             if self.spline_kind == "step_func":
                 arg = {"_step_func_coeff": True}
                 coeffs_list.append(
@@ -306,19 +363,21 @@ class Processor(object):
         return np.array(coeffs_list)
 
     def set_all_tlist(self, tlist):
-        # TODO add tests
         """
-        Set `tlist` for all the pulses. It can be used to set `tlist` if
-        all pulses are controlled by the same time sequence.
+        Set the same `tlist` for all the pulses.
 
         Parameters
         ----------
         tlist: array-like, optional
             A list of time at which the time-dependent coefficients are
-            applied. See :class:`qutip.qip.Pulse` for detailed information`
+            applied. See :class:`.Pulse` for detailed information`
         """
-        for pulse in self.pulses:
-            pulse.tlist = tlist
+        if isinstance(tlist, list) and len(tlist) == len(self.pulses):
+            for i, pulse in enumerate(self.pulses):
+                pulse.tlist = tlist[i]
+        else:
+            for pulse in self.pulses:
+                pulse.tlist = tlist
 
     def add_pulse(self, pulse):
         """
@@ -326,7 +385,7 @@ class Processor(object):
 
         Parameters
         ----------
-        pulse: :class:`qutip.qip.Pulse`
+        pulse: :class:`.Pulse`
             `Pulse` object to be added.
         """
         if isinstance(pulse, Pulse):
@@ -336,7 +395,7 @@ class Processor(object):
         else:
             raise ValueError("Invalid input, pulse must be a Pulse object")
 
-    def remove_pulse(self, indices):
+    def remove_pulse(self, indices=None, label=None):
         """
         Remove the control pulse with given indices.
 
@@ -344,12 +403,19 @@ class Processor(object):
         ----------
         indices: int or list of int
             The indices of the control Hamiltonians to be removed.
+        label: str
+            The label of the pulse
         """
-        if not isinstance(indices, Iterable):
-            indices = [indices]
-        indices.sort(reverse=True)
-        for ind in indices:
-            del self.pulses[ind]
+        if indices is not None:
+            if not isinstance(indices, Iterable):
+                indices = [indices]
+            indices.sort(reverse=True)
+            for ind in indices:
+                del self.pulses[ind]
+        else:
+            for ind, pulse in enumerate(self.pulses):
+                if pulse.label == label:
+                    del self.pulses[ind]
 
     def _is_pulses_valid(self):
         """
@@ -398,7 +464,7 @@ class Processor(object):
 
         Parameters
         ----------
-        noise: :class:`qutip.qip.Noise`
+        noise: :class:`.Noise`
             The noise object defined outside the processor
         """
         if isinstance(noise, Noise):
@@ -419,8 +485,6 @@ class Processor(object):
             True if the time list should be included in the first column.
         """
         self._is_pulses_valid()
-        # TODO this works only for step_func
-        # TODO replace this by get_complete_coeffs
         coeffs = np.array(self.get_full_coeffs())
         if inctime:
             shp = coeffs.T.shape
@@ -482,10 +546,9 @@ class Processor(object):
 
         Returns
         -------
-        noisy_pulses: list of :class"`qutip.qip.Pulse`/:class:`qutip.qip.Drift`
+        noisy_pulses: list of :class:`.Pulse`
             A list of noisy pulses.
         """
-        # TODO add tests
         pulses = deepcopy(self.pulses)
         noisy_pulses = process_noise(
             pulses, self.noise, self.dims, t1=self.t1, t2=self.t2,
@@ -544,6 +607,13 @@ class Processor(object):
         final_qu = _merge_qobjevo(qu_list)
         final_qu.args.update(args)
 
+        # bring all c_ops to the same tlist, won't need it in QuTiP 5
+        full_tlist = self.get_full_tlist()
+        temp = []
+        for c_op in c_ops:
+            temp.append(_merge_qobjevo([c_op], full_tlist))
+        c_ops = temp
+
         if noisy:
             return final_qu, c_ops
         else:
@@ -558,7 +628,7 @@ class Processor(object):
 
         Parameters
         ----------
-        qc: :class:`qutip.qip.QubitCircuit`, optional
+        qc: :class:`.QubitCircuit`, optional
             Takes the quantum circuit to be implemented. If not given, use
             the quantum circuit saved in the processor by ``load_circuit``.
 
@@ -571,14 +641,13 @@ class Processor(object):
             An instance of the class
             :class:`qutip.Result` will be returned.
         """
-        # TODO change init_state to init_state
         if init_state is not None:
             U_list = [init_state]
         else:
             U_list = []
         tlist = self.get_full_tlist()
         # TODO replace this by get_complete_coeff
-        coeffs = np.array(self.coeffs)
+        coeffs = self.get_full_coeffs()
         for n in range(len(tlist)-1):
             H = sum([coeffs[m, n] * self.ctrls[m]
                     for m in range(len(self.ctrls))])
@@ -602,7 +671,7 @@ class Processor(object):
 
         Parameters
         ----------
-        qc: :class:`qutip.qip.QubitCircuit`, optional
+        qc: :class:`.QubitCircuit`, optional
             Takes the quantum circuit to be implemented. If not given, use
             the quantum circuit saved in the processor by `load_circuit`.
 
@@ -616,15 +685,16 @@ class Processor(object):
         return self.run_analytically(qc=qc, init_state=None)
 
     def run_state(self, init_state=None, analytical=False, states=None,
-                  noisy=True, **kwargs):
+                  noisy=True, solver="mesolve", **kwargs):
         """
         If `analytical` is False, use :func:`qutip.mesolve` to
         calculate the time of the state evolution
         and return the result. Other arguments of mesolve can be
         given as keyword arguments.
+
         If `analytical` is True, calculate the propagator
         with matrix exponentiation and return a list of matrices.
-        Noise will be neglected in this choice.
+        Noise will be neglected in this option.
 
         Parameters
         ----------
@@ -637,6 +707,9 @@ class Processor(object):
         states: :class:`qutip.Qobj`, optional
             Old API, same as init_state.
 
+        solver: str
+            "mesolve" or "mcsolve"
+
         **kwargs
             Keyword arguments for the qutip solver.
 
@@ -689,14 +762,21 @@ class Processor(object):
         else:
             kwargs["c_ops"] = sys_c_ops
 
-        evo_result = mesolve(
-            H=noisy_qobjevo, rho0=init_state,
-            tlist=noisy_qobjevo.tlist, **kwargs)
+        # choose solver:
+        if solver == "mesolve":
+            evo_result = mesolve(
+                H=noisy_qobjevo, rho0=init_state,
+                tlist=noisy_qobjevo.tlist, **kwargs)
+        elif solver == "mcsolve":
+            evo_result = mcsolve(
+                H=noisy_qobjevo, psi0=init_state,
+                tlist=noisy_qobjevo.tlist, **kwargs)
+
         return evo_result
 
     def load_circuit(self, qc):
         """
-        Translate an :class:`qutip.qip.QubitCircuit` to its
+        Translate an :class:`.QubitCircuit` to its
         corresponding Hamiltonians. (Defined in subclasses)
         """
         raise NotImplementedError("Use the function in the sub-class")
@@ -708,23 +788,44 @@ class Processor(object):
         """
         return U
 
-    def plot_pulses(self, title=None, figsize=None, dpi=None):
+    def get_operators_labels(self):
         """
-        Plot the pulse amplitude
+        Get the labels for each Hamiltonian.
+        It is used in the method``plot_pulses``.
+        It is a 2-d nested list, in the plot,
+        a different color will be used for each sublist.
+        """
+        label_list = []
+        for pulse in self.pulses:
+            label_list.append(pulse.label)
+        return [label_list]
+
+    def plot_pulses(
+            self, title=None, figsize=(12, 6), dpi=None,
+            show_axis=False, rescale_pulse_coeffs=True,
+            num_steps=1000):
+        """
+        Plot the ideal pulse coefficients.
 
         Parameters
         ----------
-        noisy: bool, optional
-            If true, plot the noisy pulses.
-
-        title: str
+        title: str, optional
             Title for the plot.
 
-        figsize: tuple
-            The size of the figure
+        figsize: tuple, optional
+            The size of the figure.
 
-        dpi: int
-            The dpi of the figure
+        dpi: int, optional
+            The dpi of the figure.
+
+        show_axis: bool, optional
+            If the axis are shown.
+
+        rescale_pulse_coeffs: bool, optional
+            Rescale the hight of each pulses.
+
+        num_steps: int, optional
+            Number of time steps in the plot.
 
         Returns
         -------
@@ -739,32 +840,78 @@ class Processor(object):
         ``plot_pulses`` only works for array_like coefficients
         """
         import matplotlib.pyplot as plt
+        import matplotlib.gridspec as gridspec
+        color_list = plt.rcParams['axes.prop_cycle'].by_key()['color']
 
-        fig, ax = plt.subplots(1, 1, figsize=figsize, dpi=dpi)
-        ax.set_ylabel("Control pulse amplitude")
-        ax.set_xlabel("Time")
+        # create a axis for each pulse
+        fig = plt.figure(figsize=figsize, dpi=dpi)
+        grids = gridspec.GridSpec(len(self.pulses), 1)
+        grids.update(wspace=0., hspace=0.)
 
-        # TODO add test
-        coeffs = self.coeffs
-        tlist = self.get_full_tlist()
+        tlist = np.linspace(0., self.get_full_tlist()[-1], num_steps)
+        dt = tlist[1] - tlist[0]
 
-        for i in range(len(coeffs)):
-            if not isinstance(coeffs[i], (Iterable, np.ndarray)):
-                raise ValueError(
-                    "plot_pulse only accepts array_like coefficients.")
-            if self.spline_kind == "step_func":
-                if len(coeffs[i]) == len(tlist) - 1:
-                    coeffs[i] = np.hstack(
-                        [coeffs[i], coeffs[i][-1:]])
+        # make sure coeffs start and end with zero, for ax.fill
+        tlist = np.hstack(([-dt*1.e-20], tlist, [tlist[-1] + dt*1.e-20]))
+        coeffs = []
+        for pulse in self.pulses:
+            coeffs.append(_pulse_interpolate(pulse, tlist))
+
+        pulse_ind = 0
+        axis = []
+        for i, label_group in enumerate(self.get_operators_labels()):
+            for j, label in enumerate(label_group):
+                grid = grids[pulse_ind]
+                ax = plt.subplot(grid)
+                axis.append(ax)
+                ax.fill(tlist, coeffs[pulse_ind], color_list[i], alpha=0.7)
+                ax.plot(tlist, coeffs[pulse_ind], color_list[i])
+                if rescale_pulse_coeffs:
+                    ymax = np.max(np.abs(coeffs[pulse_ind])) * 1.1
                 else:
-                    coeffs[i][-1] = coeffs[i][-2]
-                ax.step(tlist, coeffs[i], where='post')
-            elif self.spline_kind == "cubic":
-                sp = CubicSpline(tlist, coeffs[i])
-                t_line = np.linspace(tlist[0], tlist[-1], 200)
-                c_line = [sp(t) for t in t_line]
-                ax.plot(t_line, c_line)
-        if title is not None:
-            ax.set_title(title)
+                    ymax = np.max(np.abs(coeffs)) * 1.1
+                if ymax != 0.:
+                    ax.set_ylim((-ymax, ymax))
+
+                # disable frame and ticks
+                if not show_axis:
+                    ax.set_xticks([])
+                    ax.spines['bottom'].set_visible(False)
+                ax.spines['top'].set_visible(False)
+                ax.spines['right'].set_visible(False)
+                ax.spines['left'].set_visible(False)
+                ax.set_yticks([])
+                ax.set_ylabel(label, rotation=0)
+                pulse_ind += 1
+                if i == 0 and j == 0 and title is not None:
+                    ax.set_title(title)
         fig.tight_layout()
-        return fig, ax
+        return fig, axis
+
+
+def _pulse_interpolate(pulse, tlist):
+    """
+    A function that calls Scipy interpolation routine. Used for plotting.
+    """
+    if pulse.tlist is None and pulse.coeff is None:
+        coeff = np.zeros(len(tlist))
+        return coeff
+    if isinstance(pulse.coeff, bool):
+        if pulse.coeff:
+            coeff = np.ones(len(tlist))
+        else:
+            coeff = np.zeros(len(tlist))
+        return coeff
+    coeff = pulse.coeff
+    if len(coeff) == len(pulse.tlist)-1:  # for discrete pulse
+        coeff = np.concatenate([coeff, [0]])
+
+    from scipy import interpolate
+    if pulse.spline_kind == "step_func":
+        kind = "previous"
+    else:
+        kind = "cubic"
+    inter = interpolate.interp1d(
+        pulse.tlist, coeff, kind=kind,
+        bounds_error=False, fill_value=0.0)
+    return inter(tlist)
