@@ -30,14 +30,15 @@
 #    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
+from copy import deepcopy
+
 import numpy as np
-from qutip.operators import sigmax, sigmay, sigmaz, identity
+
+from qutip.operators import sigmax, sigmay, sigmaz
 from qutip.tensor import tensor
 from qutip.qip.circuit import QubitCircuit
-from qutip.qip.device.processor import Processor
 from qutip.qip.device.modelprocessor import ModelProcessor
 from qutip.qip.pulse import Pulse
-from qutip.qip.compiler.gatecompiler import GateCompiler
 from qutip.qip.compiler.spinchaincompiler import SpinChainCompiler
 
 
@@ -52,10 +53,10 @@ class SpinChain(ModelProcessor):
     The processor can simulate the evolution under the given
     control pulses either numerically or analytically.
     It is a base class and should not be used directly, please
-    refer the the subclasses :class:`qutip.qip.LinearSpinChain` and
-    :class:`qutip.qip.CircularSpinChain`.
+    refer the the subclasses :class:`qutip.qip.device.LinearSpinChain` and
+    :class:`qutip.qip.device.CircularSpinChain`.
     (Only additional attributes are documented here, for others please
-    refer to the parent class :class:`qutip.qip.device.ModelProcessor`)
+    refer to the parent class :class:`.ModelProcessor`)
 
     Parameters
     ----------
@@ -132,23 +133,35 @@ class SpinChain(ModelProcessor):
         N: int
             The number of qubits in the system.
         """
+        self.pulse_dict = {}
+        index = 0
         # sx_ops
         for m in range(N):
             self.pulses.append(
                 Pulse(sigmax(), m, spline_kind=self.spline_kind))
+            self.pulse_dict["sx" + str(m)] = index
+            index += 1
         # sz_ops
         for m in range(N):
             self.pulses.append(
                 Pulse(sigmaz(), m, spline_kind=self.spline_kind))
+            self.pulse_dict["sz" + str(m)] = index
+            index += 1
         # sxsy_ops
         operator = tensor([sigmax(), sigmax()]) + tensor([sigmay(), sigmay()])
         for n in range(N - 1):
             self.pulses.append(
                 Pulse(operator, [n, n+1], spline_kind=self.spline_kind))
+            self.pulse_dict["g" + str(n)] = index
+            index += 1
 
     def set_up_params(self, sx, sz):
         """
         Save the parameters in the attribute `params` and check the validity.
+        The keys of `params` including "sx", "sz", and "sxsy", each
+        mapped to a list for parameters corresponding to each qubits.
+        For coupling strength "sxsy", list element i is the interaction
+        between qubits i and i+1.
 
         Parameters
         ----------
@@ -163,10 +176,10 @@ class SpinChain(ModelProcessor):
         The coefficient of sxsy is defined in the submethods.
         All parameters will be multiplied by 2*pi for simplicity
         """
-        sx_para = super(SpinChain, self)._para_list(sx, self.N)
-        self._paras["sx"] = sx_para
-        sz_para = super(SpinChain, self)._para_list(sz, self.N)
-        self._paras["sz"] = sz_para
+        sx_para = 2 * np.pi * self.to_array(sx, self.N)
+        self._params["sx"] = sx_para
+        sz_para = 2 * np.pi * self.to_array(sz, self.N)
+        self._params["sz"] = sz_para
 
     @property
     def sx_ops(self):
@@ -192,18 +205,19 @@ class SpinChain(ModelProcessor):
     def sxsy_u(self):
         return self.coeffs[2*self.N:]
 
-    def load_circuit(self, qc, setup):
+    def load_circuit(
+            self, qc, setup, schedule_mode="ASAP", compiler=None):
         """
-        Decompose a :class:`qutip.QubitCircuit` in to the control
+        Decompose a :class:`.QubitCircuit` in to the control
         amplitude generating the corresponding evolution.
 
         Parameters
         ----------
-        qc: :class:`qutip.QubitCircuit`
+        qc: :class:`.QubitCircuit`
             Takes the quantum circuit to be implemented.
 
         setup: string
-            "linear" or "circular" for two sub-calsses.
+            "linear" or "circular" for two sub-classes.
 
         Returns
         -------
@@ -216,14 +230,16 @@ class SpinChain(ModelProcessor):
             one Hamiltonian.
         """
         gates = self.optimize_circuit(qc).gates
-
-        dec = SpinChainCompiler(
-            self.N, self._paras, setup=setup,
-            global_phase=0., num_ops=len(self.ctrls))
-        tlist, self.coeffs, self.global_phase = dec.decompose(gates)
-        for i in range(len(self.pulses)):
-            self.pulses[i].tlist = tlist
-
+        if compiler is None:
+            compiler = SpinChainCompiler(
+                self.N, self._params, setup=setup,
+                global_phase=0., pulse_dict=deepcopy(self.pulse_dict))
+        tlist, coeffs = compiler.compile(
+            gates, schedule_mode=schedule_mode)
+        self.global_phase = compiler.global_phase
+        self.coeffs = coeffs
+        for i in range(len(coeffs)):
+            self.pulses[i].tlist = tlist[i]
         return tlist, self.coeffs
 
     def adjacent_gates(self, qc, setup="linear"):
@@ -234,7 +250,7 @@ class SpinChain(ModelProcessor):
 
         Parameters
         ----------
-        qc: :class:`qutip.QubitCircuit`
+        qc: :class:`.QubitCircuit`
             The circular spin chain circuit to be resolved
 
         setup: Boolean
@@ -242,7 +258,7 @@ class SpinChain(ModelProcessor):
 
         Returns
         -------
-        qc: :class:`qutip.QubitCircuit`
+        qc: :class:`.QubitCircuit`
             Returns QubitCircuit of resolved gates for the qubit circuit in the
             desired basis.
         """
@@ -446,12 +462,12 @@ class SpinChain(ModelProcessor):
 
         Parameters
         ----------
-        qc: :class:`qutip.QubitCircuit`
+        qc: :class:`.QubitCircuit`
             Takes the quantum circuit to be implemented.
 
         Returns
         -------
-        qc: :class:`qutip.QubitCircuit`
+        qc: :class:`.QubitCircuit`
             The circuit representation with elementary gates
             that can be implemented in this model.
         """
@@ -512,8 +528,8 @@ class LinearSpinChain(SpinChain):
     def set_up_params(self, sx, sz, sxsy):
         # Doc same as in the parent class
         super(LinearSpinChain, self).set_up_params(sx, sz)
-        sxsy_para = self._para_list(sxsy, self.N-1)
-        self._paras["sxsy"] = sxsy_para
+        sxsy_para = 2 * np.pi * self.to_array(sxsy, self.N-1)
+        self._params["sxsy"] = sxsy_para
 
     @property
     def sxsy_ops(self):
@@ -523,14 +539,23 @@ class LinearSpinChain(SpinChain):
     def sxsy_u(self):
         return self.coeffs[2*self.N: 3*self.N-1]
 
-    def load_circuit(self, qc):
-        return super(LinearSpinChain, self).load_circuit(qc, "linear")
+    def load_circuit(
+            self, qc, schedule_mode="ASAP", compiler=None):
+        return super(LinearSpinChain, self).load_circuit(
+            qc, "linear", schedule_mode=schedule_mode, compiler=compiler)
 
-    def get_ops_labels(self):
-        return ([r"$\sigma_x^%d$" % n for n in range(self.N)] +
-                [r"$\sigma_z^%d$" % n for n in range(self.N)] +
+    def get_operators_labels(self):
+        """
+        Get the labels for each Hamiltonian.
+        It is used in the method``plot_pulses``.
+        It is a 2-d nested list, in the plot,
+        a different color will be used for each sublist.
+        """
+        return ([[r"$\sigma_x^%d$" % n for n in range(self.N)],
+                [r"$\sigma_z^%d$" % n for n in range(self.N)],
                 [r"$\sigma_x^%d\sigma_x^{%d} + \sigma_y^%d\sigma_y^{%d}$"
-                 % (n, n, n + 1, n + 1) for n in range(self.N - 1)])
+                 % (n, n + 1, n, n + 1) for n in range(self.N - 1)],
+                 ])
 
     def adjacent_gates(self, qc):
         return super(LinearSpinChain, self).adjacent_gates(qc, "linear")
@@ -573,7 +598,10 @@ class CircularSpinChain(SpinChain):
     """
     def __init__(self, N, correct_global_phase=True,
                  sx=0.25, sz=1.0, sxsy=0.1, t1=None, t2=None):
-
+        if N <= 1:
+            raise ValueError(
+                "Circuit spin chain must have at least 2 qubits. "
+                "The number of qubits is increased to 2.")
         super(CircularSpinChain, self).__init__(
             N, correct_global_phase=correct_global_phase,
             sx=sx, sz=sz, sxsy=sxsy, t1=t1, t2=t2)
@@ -585,12 +613,13 @@ class CircularSpinChain(SpinChain):
         operator = tensor([sigmax(), sigmax()]) + tensor([sigmay(), sigmay()])
         self.pulses.append(
             Pulse(operator, [N-1, 0], spline_kind=self.spline_kind))
+        self.pulse_dict["g" + str(N-1)] = len(self.pulses) - 1
 
     def set_up_params(self, sx, sz, sxsy):
         # Doc same as in the parent class
         super(CircularSpinChain, self).set_up_params(sx, sz)
-        sxsy_para = self._para_list(sxsy, self.N)
-        self._paras["sxsy"] = sxsy_para
+        sxsy_para = 2 * np.pi * self.to_array(sxsy, self.N)
+        self._params["sxsy"] = sxsy_para
 
     @property
     def sxsy_ops(self):
@@ -600,15 +629,23 @@ class CircularSpinChain(SpinChain):
     def sxsy_u(self):
         return self.coeffs[2*self.N: 3*self.N]
 
-    def load_circuit(self, qc):
-        return super(CircularSpinChain, self).load_circuit(qc, "circular")
+    def load_circuit(
+            self, qc, schedule_mode="ASAP", compiler=None):
+        return super(CircularSpinChain, self).load_circuit(
+            qc, "circular", schedule_mode=schedule_mode, compiler=compiler)
 
-    def get_ops_labels(self):
-        return ([r"$\sigma_x^%d$" % n for n in range(self.N)] +
-                [r"$\sigma_z^%d$" % n for n in range(self.N)] +
+    def get_operators_labels(self):
+        """
+        Get the labels for each Hamiltonian.
+        It is used in the method``plot_pulses``.
+        It is a 2-d nested list, in the plot,
+        a different color will be used for each sublist.
+        """
+        return ([[r"$\sigma_x^%d$" % n for n in range(self.N)],
+                [r"$\sigma_z^%d$" % n for n in range(self.N)],
                 [r"$\sigma_x^%d\sigma_x^{%d} + \sigma_y^%d\sigma_y^{%d}$"
-                 % (n, n, (n + 1) % self.N, (n + 1) % self.N)
-                 for n in range(self.N)])
+                 % (n, (n + 1) % self.N, n, (n + 1) % self.N)
+                 for n in range(self.N)]])
 
     def adjacent_gates(self, qc):
         return super(CircularSpinChain, self).adjacent_gates(qc, "circular")
