@@ -32,20 +32,21 @@
 ###############################################################################
 import numbers
 from collections.abc import Iterable
-from itertools import product
+from itertools import product, chain
 from functools import partial, reduce
 from operator import mul
 
 import numpy as np
 import scipy.sparse as sp
 from qutip.qobj import Qobj
-from qutip.operators import identity, qeye, sigmax
+from qutip.operators import identity, qeye, sigmax, sigmay, sigmaz
 from qutip.tensor import tensor
 from qutip.states import fock_dm
 
 
 __all__ = ['rx', 'ry', 'rz', 'sqrtnot', 'snot', 'phasegate', 'qrot',
-           'cphase', 'cnot',
+           'x_gate', 'y_gate', 'z_gate', 'cy_gate', 'cz_gate', 's_gate',
+           't_gate', 'qasmu_gate', 'cs_gate', 'ct_gate', 'cphase', 'cnot',
            'csign', 'berkeley', 'swapalpha', 'swap', 'iswap', 'sqrtswap',
            'sqrtiswap', 'fredkin', 'molmer_sorensen',
            'toffoli', 'rotation', 'controlled_gate',
@@ -58,6 +59,166 @@ __all__ = ['rx', 'ry', 'rz', 'sqrtnot', 'snot', 'phasegate', 'qrot',
 #
 
 
+def x_gate(N=None, target=0):
+    """Pauli-X gate or sigmax operator.
+
+    Returns
+    -------
+    result : :class:`qutip.Qobj`
+        Quantum object for operator describing
+        a single-qubit rotation through pi radians around the x-axis.
+
+    """
+    if N is not None:
+        return gate_expand_1toN(x_gate(), N, target)
+    return sigmax()
+
+
+def y_gate(N=None, target=0):
+    """Pauli-Y gate or sigmay operator.
+
+    Returns
+    -------
+    result : :class:`qutip.Qobj`
+        Quantum object for operator describing
+        a single-qubit rotation through pi radians around the y-axis.
+
+    """
+    if N is not None:
+        return gate_expand_1toN(y_gate(), N, target)
+    return sigmay()
+
+
+def cy_gate(N=None, control=0, target=1):
+    """Controlled Y gate.
+
+    Returns
+    -------
+    result : :class:`qutip.Qobj`
+        Quantum object for operator describing the rotation.
+
+    """
+    if (control == 1 and target == 0) and N is None:
+        N = 2
+
+    if N is not None:
+        return gate_expand_2toN(cy_gate(), N, control, target)
+    return Qobj([[1, 0, 0, 0],
+                 [0, 1, 0, 0],
+                 [0, 0, 0, -1j],
+                 [0, 0, 1j, 0]],
+                dims=[[2, 2], [2, 2]])
+
+
+def z_gate(N=None, target=0):
+    """Pauli-Z gate or sigmaz operator.
+
+    Returns
+    -------
+    result : :class:`qutip.Qobj`
+        Quantum object for operator describing
+        a single-qubit rotation through pi radians around the z-axis.
+
+    """
+    if N is not None:
+        return gate_expand_1toN(z_gate(), N, target)
+    return sigmaz()
+
+
+def cz_gate(N=None, control=0, target=1):
+    """Controlled Z gate.
+
+    Returns
+    -------
+    result : :class:`qutip.Qobj`
+        Quantum object for operator describing the rotation.
+
+    """
+    if (control == 1 and target == 0) and N is None:
+        N = 2
+
+    if N is not None:
+        return gate_expand_2toN(cz_gate(), N, control, target)
+    return Qobj([[1, 0, 0, 0],
+                 [0, 1, 0, 0],
+                 [0, 0, 1, 0],
+                 [0, 0, 0, -1]],
+                dims=[[2, 2], [2, 2]])
+
+
+def s_gate(N=None, target=0):
+    """Single-qubit rotation also called Phase gate or the Z90 gate.
+
+    Returns
+    -------
+    result : :class:`qutip.Qobj`
+        Quantum object for operator describing
+        a 90 degree rotation around the z-axis.
+
+    """
+    if N is not None:
+        return gate_expand_1toN(s_gate(), N, target)
+    return Qobj([[1, 0],
+                 [0, 1j]])
+
+
+def cs_gate(N=None, control=0, target=1):
+    """Controlled S gate.
+
+    Returns
+    -------
+    result : :class:`qutip.Qobj`
+        Quantum object for operator describing the rotation.
+
+    """
+    if (control == 1 and target == 0) and N is None:
+        N = 2
+
+    if N is not None:
+        return gate_expand_2toN(cs_gate(), N, control, target)
+    return Qobj([[1, 0, 0, 0],
+                 [0, 1, 0, 0],
+                 [0, 0, 1, 0],
+                 [0, 0, 0, 1j]],
+                dims=[[2, 2], [2, 2]])
+
+
+def t_gate(N=None, target=0):
+    """Single-qubit rotation related to the S gate by the relationship S=T*T.
+
+    Returns
+    -------
+    result : :class:`qutip.Qobj`
+        Quantum object for operator describing a phase shift of pi/4.
+
+    """
+    if N is not None:
+        return gate_expand_1toN(t_gate(), N, target)
+    return Qobj([[1, 0],
+                 [0, np.exp(1j*np.pi/4)]])
+
+
+def ct_gate(N=None, control=0, target=1):
+    """Controlled T gate.
+
+    Returns
+    -------
+    result : :class:`qutip.Qobj`
+        Quantum object for operator describing the rotation.
+
+    """
+    if (control == 1 and target == 0) and N is None:
+        N = 2
+
+    if N is not None:
+        return gate_expand_2toN(ct_gate(), N, control, target)
+    return Qobj([[1, 0, 0, 0],
+                 [0, 1, 0, 0],
+                 [0, 0, 1, 0],
+                 [0, 0, 0, np.exp(1j * np.pi / 4)]],
+                dims=[[2, 2], [2, 2]])
+
+
 def rx(phi, N=None, target=0):
     """Single-qubit rotation for operator sigmax with angle phi.
 
@@ -69,9 +230,8 @@ def rx(phi, N=None, target=0):
     """
     if N is not None:
         return gate_expand_1toN(rx(phi), N, target)
-    else:
-        return Qobj([[np.cos(phi / 2), -1j * np.sin(phi / 2)],
-                     [-1j * np.sin(phi / 2), np.cos(phi / 2)]])
+    return Qobj([[np.cos(phi / 2), -1j * np.sin(phi / 2)],
+                 [-1j * np.sin(phi / 2), np.cos(phi / 2)]])
 
 
 def ry(phi, N=None, target=0):
@@ -85,9 +245,8 @@ def ry(phi, N=None, target=0):
     """
     if N is not None:
         return gate_expand_1toN(ry(phi), N, target)
-    else:
-        return Qobj([[np.cos(phi / 2), -np.sin(phi / 2)],
-                     [np.sin(phi / 2), np.cos(phi / 2)]])
+    return Qobj([[np.cos(phi / 2), -np.sin(phi / 2)],
+                 [np.sin(phi / 2), np.cos(phi / 2)]])
 
 
 def rz(phi, N=None, target=0):
@@ -101,9 +260,8 @@ def rz(phi, N=None, target=0):
     """
     if N is not None:
         return gate_expand_1toN(rz(phi), N, target)
-    else:
-        return Qobj([[np.exp(-1j * phi / 2), 0],
-                     [0, np.exp(1j * phi / 2)]])
+    return Qobj([[np.exp(-1j * phi / 2), 0],
+                 [0, np.exp(1j * phi / 2)]])
 
 
 def sqrtnot(N=None, target=0):
@@ -117,9 +275,8 @@ def sqrtnot(N=None, target=0):
     """
     if N is not None:
         return gate_expand_1toN(sqrtnot(), N, target)
-    else:
-        return Qobj([[0.5 + 0.5j, 0.5 - 0.5j],
-                     [0.5 - 0.5j, 0.5 + 0.5j]])
+    return Qobj([[0.5 + 0.5j, 0.5 - 0.5j],
+                 [0.5 - 0.5j, 0.5 + 0.5j]])
 
 
 def snot(N=None, target=0):
@@ -132,7 +289,7 @@ def snot(N=None, target=0):
 
     Examples
     --------
-    >>> snot()
+    >>> snot() # doctest: +SKIP
     Quantum object: dims = [[2], [2]], \
 shape = [2, 2], type = oper, isHerm = True
     Qobj data =
@@ -142,9 +299,8 @@ shape = [2, 2], type = oper, isHerm = True
     """
     if N is not None:
         return gate_expand_1toN(snot(), N, target)
-    else:
-        return 1 / np.sqrt(2.0) * Qobj([[1, 1],
-                                        [1, -1]])
+    return 1 / np.sqrt(2.0) * Qobj([[1, 1],
+                                    [1, -1]])
 
 
 def phasegate(theta, N=None, target=0):
@@ -163,7 +319,7 @@ def phasegate(theta, N=None, target=0):
 
     Examples
     --------
-    >>> phasegate(pi/4)
+    >>> phasegate(pi/4) # doctest: +SKIP
     Quantum object: dims = [[2], [2]], \
 shape = [2, 2], type = oper, isHerm = False
     Qobj data =
@@ -173,10 +329,9 @@ shape = [2, 2], type = oper, isHerm = False
     """
     if N is not None:
         return gate_expand_1toN(phasegate(theta), N, target)
-    else:
-        return Qobj([[1, 0],
-                     [0, np.exp(1.0j * theta)]],
-                    dims=[[2], [2]])
+    return Qobj([[1, 0],
+                 [0, np.exp(1.0j * theta)]],
+                dims=[[2], [2]])
 
 
 def qrot(theta, phi, N=None, target=0):
@@ -202,12 +357,43 @@ def qrot(theta, phi, N=None, target=0):
     """
     if N is not None:
         return expand_operator(qrot(theta, phi), N=N, targets=target)
-    else:
-        return Qobj(
-            [
-                [np.cos(theta/2.), -1.j*np.exp(-1.j*phi)*np.sin(theta/2.)],
-                [-1.j*np.exp(1.j*phi)*np.sin(theta/2.), np.cos(theta/2.)]
-            ])
+    return Qobj(
+        [
+            [np.cos(theta/2.), -1.j*np.exp(-1.j*phi)*np.sin(theta/2.)],
+            [-1.j*np.exp(1.j*phi)*np.sin(theta/2.), np.cos(theta/2.)]
+        ])
+
+
+def qasmu_gate(args, N=None, target=0):
+    """
+    QASM U-gate as defined in the OpenQASM standard.
+
+    Parameters
+    ----------
+
+    theta : float
+        The argument supplied to the last RZ rotation.
+    phi : float
+        The argument supplied to the middle RY rotation.
+    gamma : float
+        The argument supplied to the first RZ rotation.
+    N : int
+        Number of qubits in the system.
+    target : int
+        The index of the target qubit.
+
+    Returns
+    -------
+    qasmu_gate : :class:`qutip.Qobj`
+        Quantum object representation of the QASM U-gate as defined in the
+        OpenQASM standard.
+    """
+
+    theta, phi, gamma = args
+    if N is not None:
+        return expand_operator(qasmu_gate([theta, phi, gamma]), N=N,
+                               targets=target)
+    return Qobj(rz(phi) * ry(theta) * rz(gamma))
 
 
 #
@@ -267,7 +453,7 @@ def cnot(N=None, control=0, target=1):
 
     Examples
     --------
-    >>> cnot()
+    >>> cnot() # doctest: +SKIP
     Quantum object: dims = [[2, 2], [2, 2]], \
 shape = [4, 4], type = oper, isHerm = True
     Qobj data =
@@ -282,12 +468,11 @@ shape = [4, 4], type = oper, isHerm = True
 
     if N is not None:
         return gate_expand_2toN(cnot(), N, control, target)
-    else:
-        return Qobj([[1, 0, 0, 0],
-                     [0, 1, 0, 0],
-                     [0, 0, 0, 1],
-                     [0, 0, 1, 0]],
-                    dims=[[2, 2], [2, 2]])
+    return Qobj([[1, 0, 0, 0],
+                 [0, 1, 0, 0],
+                 [0, 0, 0, 1],
+                 [0, 0, 1, 0]],
+                dims=[[2, 2], [2, 2]])
 
 
 def csign(N=None, control=0, target=1):
@@ -301,7 +486,7 @@ def csign(N=None, control=0, target=1):
 
     Examples
     --------
-    >>> csign()
+    >>> csign() # doctest: +SKIP
     Quantum object: dims = [[2, 2], [2, 2]], \
 shape = [4, 4], type = oper, isHerm = True
     Qobj data =
@@ -316,12 +501,11 @@ shape = [4, 4], type = oper, isHerm = True
 
     if N is not None:
         return gate_expand_2toN(csign(), N, control, target)
-    else:
-        return Qobj([[1, 0, 0, 0],
-                     [0, 1, 0, 0],
-                     [0, 0, 1, 0],
-                     [0, 0, 0, -1]],
-                    dims=[[2, 2], [2, 2]])
+    return Qobj([[1, 0, 0, 0],
+                 [0, 1, 0, 0],
+                 [0, 0, 1, 0],
+                 [0, 0, 0, -1]],
+                dims=[[2, 2], [2, 2]])
 
 
 def berkeley(N=None, targets=[0, 1]):
@@ -335,7 +519,7 @@ def berkeley(N=None, targets=[0, 1]):
 
     Examples
     --------
-    >>> berkeley()
+    >>> berkeley() # doctest: +SKIP
     Quantum object: dims = [[2, 2], [2, 2]], \
 shape = [4, 4], type = oper, isHerm = True
     Qobj data =
@@ -349,15 +533,14 @@ shape = [4, 4], type = oper, isHerm = True
         N = 2
 
     if N is not None:
-        return gate_expand_2toN(cnot(), N, targets=targets)
-    else:
-        return Qobj([[np.cos(np.pi / 8), 0, 0, 1.0j * np.sin(np.pi / 8)],
-                     [0, np.cos(3 * np.pi / 8), 1.0j *
-                      np.sin(3 * np.pi / 8), 0],
-                     [0, 1.0j * np.sin(3 * np.pi / 8),
-                      np.cos(3 * np.pi / 8), 0],
-                     [1.0j * np.sin(np.pi / 8), 0, 0, np.cos(np.pi / 8)]],
-                    dims=[[2, 2], [2, 2]])
+        return gate_expand_2toN(berkeley(), N, targets=targets)
+    return Qobj([[np.cos(np.pi / 8), 0, 0, 1.0j * np.sin(np.pi / 8)],
+                 [0, np.cos(3 * np.pi / 8), 1.0j *
+                  np.sin(3 * np.pi / 8), 0],
+                 [0, 1.0j * np.sin(3 * np.pi / 8),
+                  np.cos(3 * np.pi / 8), 0],
+                 [1.0j * np.sin(np.pi / 8), 0, 0, np.cos(np.pi / 8)]],
+                dims=[[2, 2], [2, 2]])
 
 
 def swapalpha(alpha, N=None, targets=[0, 1]):
@@ -371,7 +554,7 @@ def swapalpha(alpha, N=None, targets=[0, 1]):
 
     Examples
     --------
-    >>> swapalpha(alpha)
+    >>> swapalpha(alpha) # doctest: +SKIP
     Quantum object: dims = [[2, 2], [2, 2]], \
 shape = [4, 4], type = oper, isHerm = True
     Qobj data =
@@ -385,15 +568,14 @@ shape = [4, 4], type = oper, isHerm = True
         N = 2
 
     if N is not None:
-        return gate_expand_2toN(cnot(), N, targets=targets)
-    else:
-        return Qobj([[1, 0, 0, 0],
-                     [0, 0.5 * (1 + np.exp(1.0j * np.pi * alpha)),
-                      0.5 * (1 - np.exp(1.0j * np.pi * alpha)), 0],
-                     [0, 0.5 * (1 - np.exp(1.0j * np.pi * alpha)),
-                      0.5 * (1 + np.exp(1.0j * np.pi * alpha)), 0],
-                     [0, 0, 0, 1]],
-                    dims=[[2, 2], [2, 2]])
+        return gate_expand_2toN(swapalpha(alpha), N, targets=targets)
+    return Qobj([[1, 0, 0, 0],
+                 [0, 0.5 * (1 + np.exp(1.0j * np.pi * alpha)),
+                  0.5 * (1 - np.exp(1.0j * np.pi * alpha)), 0],
+                 [0, 0.5 * (1 - np.exp(1.0j * np.pi * alpha)),
+                  0.5 * (1 + np.exp(1.0j * np.pi * alpha)), 0],
+                 [0, 0, 0, 1]],
+                dims=[[2, 2], [2, 2]])
 
 
 def swap(N=None, targets=[0, 1]):
@@ -406,7 +588,7 @@ def swap(N=None, targets=[0, 1]):
 
     Examples
     --------
-    >>> swap()
+    >>> swap() # doctest: +SKIP
     Quantum object: dims = [[2, 2], [2, 2]], \
 shape = [4, 4], type = oper, isHerm = True
     Qobj data =
@@ -421,13 +603,11 @@ shape = [4, 4], type = oper, isHerm = True
 
     if N is not None:
         return gate_expand_2toN(swap(), N, targets=targets)
-
-    else:
-        return Qobj([[1, 0, 0, 0],
-                     [0, 0, 1, 0],
-                     [0, 1, 0, 0],
-                     [0, 0, 0, 1]],
-                    dims=[[2, 2], [2, 2]])
+    return Qobj([[1, 0, 0, 0],
+                 [0, 0, 1, 0],
+                 [0, 1, 0, 0],
+                 [0, 0, 0, 1]],
+                dims=[[2, 2], [2, 2]])
 
 
 def iswap(N=None, targets=[0, 1]):
@@ -440,7 +620,7 @@ def iswap(N=None, targets=[0, 1]):
 
     Examples
     --------
-    >>> iswap()
+    >>> iswap() # doctest: +SKIP
     Quantum object: dims = [[2, 2], [2, 2]], \
 shape = [4, 4], type = oper, isHerm = False
     Qobj data =
@@ -454,13 +634,11 @@ shape = [4, 4], type = oper, isHerm = False
 
     if N is not None:
         return gate_expand_2toN(iswap(), N, targets=targets)
-
-    else:
-        return Qobj([[1, 0, 0, 0],
-                     [0, 0, 1j, 0],
-                     [0, 1j, 0, 0],
-                     [0, 0, 0, 1]],
-                    dims=[[2, 2], [2, 2]])
+    return Qobj([[1, 0, 0, 0],
+                 [0, 0, 1j, 0],
+                 [0, 1j, 0, 0],
+                 [0, 0, 0, 1]],
+                dims=[[2, 2], [2, 2]])
 
 
 def sqrtswap(N=None, targets=[0, 1]):
@@ -477,12 +655,11 @@ def sqrtswap(N=None, targets=[0, 1]):
 
     if N is not None:
         return gate_expand_2toN(sqrtswap(), N, targets=targets)
-    else:
-        return Qobj(np.array([[1, 0, 0, 0],
-                              [0, 0.5 + 0.5j, 0.5 - 0.5j, 0],
-                              [0, 0.5 - 0.5j, 0.5 + 0.5j, 0],
-                              [0, 0, 0, 1]]),
-                    dims=[[2, 2], [2, 2]])
+    return Qobj(np.array([[1, 0, 0, 0],
+                          [0, 0.5 + 0.5j, 0.5 - 0.5j, 0],
+                          [0, 0.5 - 0.5j, 0.5 + 0.5j, 0],
+                          [0, 0, 0, 1]]),
+                dims=[[2, 2], [2, 2]])
 
 
 def sqrtiswap(N=None, targets=[0, 1]):
@@ -495,7 +672,7 @@ def sqrtiswap(N=None, targets=[0, 1]):
 
     Examples
     --------
-    >>> sqrtiswap()
+    >>> sqrtiswap() # doctest: +SKIP
     Quantum object: dims = [[2, 2], [2, 2]], \
 shape = [4, 4], type = oper, isHerm = False
     Qobj data =
@@ -514,11 +691,10 @@ shape = [4, 4], type = oper, isHerm = False
 
     if N is not None:
         return gate_expand_2toN(sqrtiswap(), N, targets=targets)
-    else:
-        return Qobj(np.array([[1, 0, 0, 0],
-                              [0, 1 / np.sqrt(2), 1j / np.sqrt(2), 0],
-                              [0, 1j / np.sqrt(2), 1 / np.sqrt(2), 0],
-                              [0, 0, 0, 1]]), dims=[[2, 2], [2, 2]])
+    return Qobj(np.array([[1, 0, 0, 0],
+                          [0, 1 / np.sqrt(2), 1j / np.sqrt(2), 0],
+                          [0, 1j / np.sqrt(2), 1 / np.sqrt(2), 0],
+                          [0, 0, 0, 1]]), dims=[[2, 2], [2, 2]])
 
 
 def molmer_sorensen(theta, N=None, targets=[0, 1]):
@@ -544,15 +720,14 @@ def molmer_sorensen(theta, N=None, targets=[0, 1]):
 
     if N is not None:
         return expand_operator(molmer_sorensen(theta), N, targets=targets)
-    else:
-        return Qobj(
-            [
-                [np.cos(theta/2.), 0, 0, -1.j*np.sin(theta/2.)],
-                [0, np.cos(theta/2.), -1.j*np.sin(theta/2.), 0],
-                [0, -1.j*np.sin(theta/2.), np.cos(theta/2.), 0],
-                [-1.j*np.sin(theta/2.), 0, 0, np.cos(theta/2.)]
-            ],
-            dims=[[2, 2], [2, 2]])
+    return Qobj(
+        [
+            [np.cos(theta/2.), 0, 0, -1.j*np.sin(theta/2.)],
+            [0, np.cos(theta/2.), -1.j*np.sin(theta/2.), 0],
+            [0, -1.j*np.sin(theta/2.), np.cos(theta/2.), 0],
+            [-1.j*np.sin(theta/2.), 0, 0, np.cos(theta/2.)]
+        ],
+        dims=[[2, 2], [2, 2]])
 
 
 #
@@ -569,7 +744,7 @@ def fredkin(N=None, control=0, targets=[1, 2]):
 
     Examples
     --------
-    >>> fredkin()
+    >>> fredkin() # doctest: +SKIP
     Quantum object: dims = [[2, 2, 2], [2, 2, 2]], \
 shape = [8, 8], type = oper, isHerm = True
     Qobj data =
@@ -589,17 +764,15 @@ shape = [8, 8], type = oper, isHerm = True
     if N is not None:
         return gate_expand_3toN(fredkin(), N,
                                 [control, targets[0]], targets[1])
-
-    else:
-        return Qobj([[1, 0, 0, 0, 0, 0, 0, 0],
-                     [0, 1, 0, 0, 0, 0, 0, 0],
-                     [0, 0, 1, 0, 0, 0, 0, 0],
-                     [0, 0, 0, 1, 0, 0, 0, 0],
-                     [0, 0, 0, 0, 1, 0, 0, 0],
-                     [0, 0, 0, 0, 0, 0, 1, 0],
-                     [0, 0, 0, 0, 0, 1, 0, 0],
-                     [0, 0, 0, 0, 0, 0, 0, 1]],
-                    dims=[[2, 2, 2], [2, 2, 2]])
+    return Qobj([[1, 0, 0, 0, 0, 0, 0, 0],
+                 [0, 1, 0, 0, 0, 0, 0, 0],
+                 [0, 0, 1, 0, 0, 0, 0, 0],
+                 [0, 0, 0, 1, 0, 0, 0, 0],
+                 [0, 0, 0, 0, 1, 0, 0, 0],
+                 [0, 0, 0, 0, 0, 0, 1, 0],
+                 [0, 0, 0, 0, 0, 1, 0, 0],
+                 [0, 0, 0, 0, 0, 0, 0, 1]],
+                dims=[[2, 2, 2], [2, 2, 2]])
 
 
 def toffoli(N=None, controls=[0, 1], target=2):
@@ -612,9 +785,9 @@ def toffoli(N=None, controls=[0, 1], target=2):
 
     Examples
     --------
-    >>> toffoli()
+    >>> toffoli() # doctest: +SKIP
     Quantum object: dims = [[2, 2, 2], [2, 2, 2]], \
-shape = [8, 8], type = oper, isHerm = True
+                    shape = [8, 8], type = oper, isHerm = True
     Qobj data =
         [[ 1.+0.j  0.+0.j  0.+0.j  0.+0.j  0.+0.j  0.+0.j  0.+0.j  0.+0.j]
          [ 0.+0.j  1.+0.j  0.+0.j  0.+0.j  0.+0.j  0.+0.j  0.+0.j  0.+0.j]
@@ -632,17 +805,15 @@ shape = [8, 8], type = oper, isHerm = True
 
     if N is not None:
         return gate_expand_3toN(toffoli(), N, controls, target)
-
-    else:
-        return Qobj([[1, 0, 0, 0, 0, 0, 0, 0],
-                     [0, 1, 0, 0, 0, 0, 0, 0],
-                     [0, 0, 1, 0, 0, 0, 0, 0],
-                     [0, 0, 0, 1, 0, 0, 0, 0],
-                     [0, 0, 0, 0, 1, 0, 0, 0],
-                     [0, 0, 0, 0, 0, 1, 0, 0],
-                     [0, 0, 0, 0, 0, 0, 0, 1],
-                     [0, 0, 0, 0, 0, 0, 1, 0]],
-                    dims=[[2, 2, 2], [2, 2, 2]])
+    return Qobj([[1, 0, 0, 0, 0, 0, 0, 0],
+                 [0, 1, 0, 0, 0, 0, 0, 0],
+                 [0, 0, 1, 0, 0, 0, 0, 0],
+                 [0, 0, 0, 1, 0, 0, 0, 0],
+                 [0, 0, 0, 0, 1, 0, 0, 0],
+                 [0, 0, 0, 0, 0, 1, 0, 0],
+                 [0, 0, 0, 0, 0, 0, 0, 1],
+                 [0, 0, 0, 0, 0, 0, 1, 0]],
+                dims=[[2, 2, 2], [2, 2, 2]])
 
 
 #
@@ -660,8 +831,7 @@ def rotation(op, phi, N=None, target=0):
     """
     if N is not None:
         return gate_expand_1toN(rotation(op, phi), N, target)
-    else:
-        return (-1j * op * phi / 2).expm()
+    return (-1j * op * phi / 2).expm()
 
 
 def controlled_gate(U, N=2, control=0, target=1, control_value=1):
@@ -696,9 +866,8 @@ def controlled_gate(U, N=2, control=0, target=1, control_value=1):
     if [N, control, target] == [2, 0, 1]:
         return (tensor(fock_dm(2, control_value), U) +
                 tensor(fock_dm(2, 1 - control_value), identity(2)))
-    else:
-        U2 = controlled_gate(U, control_value=control_value)
-        return gate_expand_2toN(U2, N=N, control=control, target=target)
+    U2 = controlled_gate(U, control_value=control_value)
+    return gate_expand_2toN(U2, N=N, control=control, target=target)
 
 
 def globalphase(theta, N=1):
@@ -717,7 +886,7 @@ def globalphase(theta, N=1):
 
     Examples
     --------
-    >>> phasegate(pi/4)
+    >>> phasegate(pi/4) # doctest: +SKIP
     Quantum object: dims = [[2], [2]], \
 shape = [2, 2], type = oper, isHerm = False
     Qobj data =
@@ -762,14 +931,213 @@ def hadamard_transform(N=1):
     return Qobj(data, dims=[[2] * N, [2] * N])
 
 
-def gate_sequence_product(U_list, left_to_right=True):
+def _flatten(lst):
+    """
+    Helper to flatten lists.
+    """
+
+    return [item for sublist in lst for item in sublist]
+
+
+def _mult_sublists(tensor_list, overall_inds, U, inds):
+    """
+    Calculate the revised indices and tensor list by multiplying a new unitary
+    U applied to inds.
+
+    Parameters
+    ----------
+    tensor_list : list of Qobj
+        List of gates (unitaries) acting on disjoint qubits.
+
+    overall_inds : list of list of int
+        List of qubit indices corresponding to each gate in tensor_list.
+
+    U: Qobj
+        Unitary to be multiplied with the the unitary specified by tensor_list.
+
+    inds: list of int
+        List of qubit indices corresponding to U.
+
+    Returns
+    -------
+    tensor_list_revised: list of Qobj
+        List of gates (unitaries) acting on disjoint qubits incorporating U.
+
+    overall_inds_revised: list of list of int
+        List of qubit indices corresponding to each gate in tensor_list_revised.
+
+    Examples
+    --------
+
+    First, we get some imports out of the way,
+
+    >>> from qutip.qip.operations.gates import _mult_sublists
+    >>> from qutip.qip.operations.gates import x_gate, y_gate, toffoli, z_gate
+
+    Suppose we have a unitary list of already processed gates,
+    X, Y, Z applied on qubit indices 0, 1, 2 respectively and
+    encounter a new TOFFOLI gate on qubit indices (0, 1, 3).
+
+    >>> tensor_list = [x_gate(), y_gate(), z_gate()]
+    >>> overall_inds = [[0], [1], [2]]
+    >>> U = toffoli()
+    >>> U_inds = [0, 1, 3]
+
+    Then, we can use _mult_sublists to produce a new list of unitaries by
+    multiplying TOFFOLI (and expanding) only on the qubit indices involving
+    TOFFOLI gate (and any multiplied gates).
+
+    >>> U_list, overall_inds = _mult_sublists(tensor_list, overall_inds, U, U_inds)
+    >>> np.testing.assert_allclose(U_list[0]) == z_gate())
+    >>> toffoli_xy = toffoli() * tensor(x_gate(), y_gate(), identity(2))
+    >>> np.testing.assert_allclose(U_list[1]), toffoli_xy)
+    >>> overall_inds = [[2], [0, 1, 3]]
+    """
+
+    tensor_sublist = []
+    inds_sublist = []
+
+    tensor_list_revised = []
+    overall_inds_revised = []
+
+    for sub_inds, sub_U in zip(overall_inds, tensor_list):
+        if len(set(sub_inds).intersection(inds)) > 0:
+            tensor_sublist.append(sub_U)
+            inds_sublist.append(sub_inds)
+        else:
+            overall_inds_revised.append(sub_inds)
+            tensor_list_revised.append(sub_U)
+
+    inds_sublist = _flatten(inds_sublist)
+    U_sublist = tensor(tensor_sublist)
+
+    revised_inds = list(set(inds_sublist).union(set(inds)))
+    N = len(revised_inds)
+
+    sorted_positions = sorted(range(N), key=lambda key: revised_inds[key])
+    ind_map = {ind: pos for ind, pos in zip(revised_inds, sorted_positions)}
+
+    U_sublist = expand_operator(U_sublist, N,
+                                [ind_map[ind] for ind in inds_sublist])
+    U = expand_operator(U, N, [ind_map[ind] for ind in inds])
+
+    U_sublist = U * U_sublist
+    inds_sublist = revised_inds
+
+    overall_inds_revised.append(inds_sublist)
+    tensor_list_revised.append(U_sublist)
+
+    return tensor_list_revised, overall_inds_revised
+
+
+def _expand_overall(tensor_list, overall_inds):
+    """
+    Tensor unitaries in tensor list and then use expand_operator to rearrange
+    them appropriately according to the indices in overall_inds.
+    """
+
+    U_overall = tensor(tensor_list)
+    overall_inds = _flatten(overall_inds)
+    U_overall = expand_operator(U_overall,
+                                len(overall_inds), overall_inds)
+    overall_inds = sorted(overall_inds)
+    return U_overall, overall_inds
+
+
+def _gate_sequence_product(U_list, ind_list):
     """
     Calculate the overall unitary matrix for a given list of unitary operations
+    that are still of original dimension.
+
+    Parameters
+    ----------
+    U_list : list of Qobj
+        List of gates(unitaries) implementing the quantum circuit.
+
+    ind_list : list of list of int
+        List of qubit indices corresponding to each gate in tensor_list.
+
+    Returns
+    -------
+    U_overall : qobj
+        Unitary matrix corresponding to U_list.
+
+    overall_inds : list of int
+        List of qubit indices on which U_overall applies.
+
+    Examples
+    --------
+
+    First, we get some imports out of the way,
+
+    >>> from qutip.qip.operations.gates import _gate_sequence_product
+    >>> from qutip.qip.operations.gates import x_gate, y_gate, toffoli, z_gate
+
+    Suppose we have a circuit with gates X, Y, Z, TOFFOLI
+    applied on qubit indices 0, 1, 2 and [0, 1, 3] respectively.
+
+    >>> tensor_lst = [x_gate(), y_gate(), z_gate(), toffoli()]
+    >>> overall_inds = [[0], [1], [2], [0, 1, 3]]
+
+    Then, we can use _gate_sequence_product to produce a single unitary
+    obtained by multiplying unitaries in the list using heuristic methods
+    to reduce the size of matrices being multiplied.
+
+    >>> U_list, overall_inds = _gate_sequence_product(tensor_lst, overall_inds)
+    """
+
+    num_qubits = len(set(chain(*ind_list)))
+    sorted_inds = sorted(set(_flatten(ind_list)))
+    ind_list = [[sorted_inds.index(ind) for ind in inds] for inds in ind_list]
+
+    U_overall = 1
+    overall_inds = []
+
+    for i, (U, inds) in enumerate(zip(U_list, ind_list)):
+
+        # when the tensor_list covers the full dimension of the circuit, we
+        # expand the tensor_list to a unitary and call _gate_sequence_product
+        # recursively on the rest of the U_list.
+        if len(overall_inds) == 1 and len(overall_inds[0]) == num_qubits:
+            U_overall, overall_inds = _expand_overall(tensor_list, overall_inds)
+            U_left, rem_inds = _gate_sequence_product(U_list[i:],
+                                                      ind_list[i:])
+            U_left = expand_operator(U_left, num_qubits, rem_inds)
+            return U_left * U_overall, [sorted_inds[ind] for ind in overall_inds]
+
+        # special case for first unitary in the list
+        if U_overall == 1:
+            U_overall = U_overall * U
+            overall_inds = [ind_list[0]]
+            tensor_list = [U_overall]
+            continue
+
+        # case where the next unitary interacts on some subset of qubits
+        # with the unitaries already in tensor_list.
+        elif len(set(_flatten(overall_inds)).intersection(set(inds))) > 0:
+            tensor_list, overall_inds = _mult_sublists(tensor_list,
+                                                       overall_inds,
+                                                       U, inds)
+
+        # case where the next unitary does not interact with any unitary in
+        # tensor_list
+        else:
+            overall_inds.append(inds)
+            tensor_list.append(U)
+
+    U_overall, overall_inds = _expand_overall(tensor_list, overall_inds)
+
+    return U_overall, [sorted_inds[ind] for ind in overall_inds]
+
+
+def _gate_sequence_product_with_expansion(U_list, left_to_right=True):
+    """
+    Calculate the overall unitary matrix for a given list of unitary operations.
 
     Parameters
     ----------
     U_list : list
-        List of gates implementing the quantum circuit.
+        List of gates(unitaries) implementing the quantum circuit.
 
     left_to_right : Boolean
         Check if multiplication is to be done from left to right.
@@ -777,9 +1145,9 @@ def gate_sequence_product(U_list, left_to_right=True):
     Returns
     -------
     U_overall : qobj
-        Overall unitary matrix of a given quantum circuit.
-
+        Unitary matrix corresponding to U_list.
     """
+
     U_overall = 1
     for U in U_list:
         if left_to_right:
@@ -790,6 +1158,41 @@ def gate_sequence_product(U_list, left_to_right=True):
     return U_overall
 
 
+def gate_sequence_product(U_list, left_to_right=True,
+                          inds_list=None, expand=False):
+    """
+    Calculate the overall unitary matrix for a given list of unitary operations.
+
+    Parameters
+    ----------
+    U_list: list
+        List of gates implementing the quantum circuit.
+
+    left_to_right: Boolean, optional
+        Check if multiplication is to be done from left to right.
+
+    inds_list: list of list of int, optional
+        If expand=True, list of qubit indices corresponding to U_list
+        to which each unitary is applied.
+
+    expand: Boolean, optional
+        Check if the list of unitaries need to be expanded to full dimension.
+
+    Returns
+    -------
+    U_overall : qobj
+        Unitary matrix corresponding to U_list.
+
+    overall_inds : list of int, optional
+        List of qubit indices on which U_overall applies.
+    """
+
+    if expand:
+        return _gate_sequence_product(U_list, inds_list)
+    else:
+        return _gate_sequence_product_with_expansion(U_list, left_to_right)
+
+
 def _powers(op, N):
     """
     Generator that yields powers of an operator `op`,
@@ -842,7 +1245,8 @@ def qubit_clifford_group(N=None, target=0):
     E = H * (S ** 3) * w ** 3
 
     for op in map(partial(reduce, mul), product(_powers(E, 3),
-                  _powers(X, 2), _powers(S, 4))):
+                                                _powers(X, 2),
+                                                _powers(S, 4))):
 
         # partial(reduce, mul) returns a function that takes products
         # of its argument, by analogy to sum. Note that by analogy,
@@ -990,8 +1394,8 @@ def gate_expand_3toN(U, N, controls=[0, 1], target=2):
                          " Must be integer < integer N")
 
     if (controls[0] == target or
-        controls[1] == target or
-        controls[0] == controls[1]):
+            controls[1] == target or
+            controls[0] == controls[1]):
 
         raise ValueError("controls[0], controls[1], and target"
                          " cannot be equal")
