@@ -51,11 +51,10 @@ except ImportError:
 from numpy import (arccos, arccosh, arcsin, arcsinh, arctan, arctan2, arctanh,
                    ceil, copysign, cos, cosh, degrees, e, exp, expm1, fabs,
                    floor, fmod, frexp, hypot, isinf, isnan, ldexp, log, log10,
-                   log1p, modf, pi, radians, sin, sinh, sqrt, tan, tanh, trunc)
+                   log1p, modf, pi, radians, sin, sinh, sqrt, tan, tanh)
 
 import numpy as np
 import scipy.sparse as sp
-import scipy.linalg as la
 import qutip.settings as settings
 from qutip import __version__
 from qutip.fastsparse import fast_csr_matrix, fast_identity
@@ -700,7 +699,7 @@ class Qobj(object):
         """
         out = self.data[ind]
         if sp.issparse(out):
-            return np.asarray(out.todense())
+            return out.toarray()
         else:
             return out
 
@@ -1016,8 +1015,8 @@ class Qobj(object):
         """
         if self.type in ['oper', 'super']:
             if norm is None or norm == 'tr':
-                _op = self*self.dag()
-                vals = sp_eigs(_op.data, _op.isherm, vecs=False,
+                _op = self.data * zcsr_adjoint(self.data)
+                vals = sp_eigs(_op, True, vecs=False,
                                sparse=sparse, tol=tol, maxiter=maxiter)
                 return np.sum(np.sqrt(np.abs(vals)))
             elif norm == 'fro':
@@ -1356,6 +1355,8 @@ class Qobj(object):
         ----------
         sel : int/list
             An ``int`` or ``list`` of components to keep after partial trace.
+            The order is unimportant; no transposition will be done and the
+            spaces will remain in the same order in the output.
 
         Returns
         -------
@@ -1587,11 +1588,10 @@ class Qobj(object):
         elem : complex
             Complex valued matrix element.
 
-        Note
-        ----
+        Notes
+        -----
         It is slightly more computationally efficient to use a ket
         vector for the 'bra' input.
-
         """
         if not self.isoper:
             raise TypeError("Can only get matrix elements for an operator.")
@@ -1674,8 +1674,8 @@ class Qobj(object):
                     raise TypeError(err)
         raise TypeError("Can only calculate overlap for state vector Qobjs")
 
-    def eigenstates(self, sparse=False, sort='low',
-                    eigvals=0, tol=0, maxiter=100000):
+    def eigenstates(self, sparse=False, sort='low', eigvals=0,
+                    tol=0, maxiter=100000, phase_fix=None):
         """Eigenstates and eigenenergies.
 
         Eigenstates and eigenenergies are defined for operators and
@@ -1699,6 +1699,10 @@ class Qobj(object):
         maxiter : int
             Maximum number of iterations performed by sparse solver (if used).
 
+        phase_fix : int, None
+            If not None, set the phase of each kets so that ket[phase_fix,0]
+            is real positive.
+
         Returns
         -------
         eigvals : array
@@ -1717,11 +1721,22 @@ class Qobj(object):
         evals, evecs = sp_eigs(self.data, self.isherm, sparse=sparse,
                                sort=sort, eigvals=eigvals, tol=tol,
                                maxiter=maxiter)
-        new_dims = [self.dims[0], [1] * len(self.dims[0])]
+        if self.type == 'super':
+            new_dims = [self.dims[0], [1]]
+            new_type = 'operator-ket'
+        else:
+            new_dims = [self.dims[0], [1] * len(self.dims[0])]
+            new_type = 'ket'
         ekets = np.empty((len(evecs),), dtype=object)
-        ekets[:] = [Qobj(vec, dims=new_dims) for vec in evecs]
+        ekets[:] = [Qobj(vec, dims=new_dims, type=new_type) for vec in evecs]
         norms = np.array([ket.norm() for ket in ekets])
-        return evals, ekets / norms
+        if phase_fix is None:
+            phase = np.array([1] * len(ekets))
+        else:
+            phase = np.array([np.abs(ket[phase_fix, 0]) / ket[phase_fix, 0]
+                              if ket[phase_fix, 0] else 1
+                              for ket in ekets])
+        return evals, ekets / norms * phase
 
     def eigenenergies(self, sparse=False, sort='low',
                       eigvals=0, tol=0, maxiter=100000):
@@ -2148,7 +2163,7 @@ class Qobj(object):
 #
 def qobj_list_evaluate(qobj_list, t, args):
     """
-    Depracated: See Qobj.evaluate
+    Deprecated: See Qobj.evaluate
     """
     warnings.warn("Deprecated: Use Qobj.evaluate", DeprecationWarning)
     return Qobj.evaluate(qobj_list, t, args)
@@ -2220,25 +2235,34 @@ def _ptrace_dense(Q, sel):
     else:
         sel = np.asarray(sel)
     sel = list(np.sort(sel))
+    for x in sel:
+        if not 0 <= x < len(rd):
+            raise IndexError("Invalid selection index in ptrace.")
     dkeep = (rd[sel]).tolist()
     qtrace = list(set(np.arange(nd)) - set(sel))
     dtrace = (rd[qtrace]).tolist()
+    if len(dkeep) + len(dtrace) != len(rd):
+        raise ValueError("Duplicate selection index in ptrace.")
+    if not dtrace:
+        # If we are keeping all dimensions, no need to construct an ndarray.
+        return Q.copy()
     rd = list(rd)
     if isket(Q):
         vmat = (Q.full()
                 .reshape(rd)
                 .transpose(sel + qtrace)
-                .reshape([np.prod(dkeep), np.prod(dtrace)]))
+                .reshape([np.prod(dkeep, dtype=np.int32),
+                          np.prod(dtrace, dtype=np.int32)]))
         rhomat = vmat.dot(vmat.conj().T)
     else:
         rhomat = np.trace(Q.full()
                           .reshape(rd + rd)
                           .transpose(qtrace + [nd + q for q in qtrace] +
                                      sel + [nd + q for q in sel])
-                          .reshape([np.prod(dtrace),
-                                    np.prod(dtrace),
-                                    np.prod(dkeep),
-                                    np.prod(dkeep)]))
+                          .reshape([np.prod(dtrace, dtype=np.int32),
+                                    np.prod(dtrace, dtype=np.int32),
+                                    np.prod(dkeep, dtype=np.int32),
+                                    np.prod(dkeep, dtype=np.int32)]))
     return Qobj(rhomat, dims=[dkeep, dkeep])
 
 
