@@ -141,14 +141,13 @@ def rand_herm(N, density=0.75, dims=None, pos_def=False, seed=None):
     oper : qobj
         NxN Hermitian quantum operator.
 
-    Note
-    ----
+    Notes
+    -----
     If given a list/ndarray as input 'N', this function returns a
     random Hermitian object with eigenvalues given in the list/ndarray.
     This is accomplished via complex Jacobi rotations.  While this method
     is ~50% faster than the corresponding (real only) Matlab code, it should
     not be repeatedly used for generating matrices larger than ~1000x1000.
-
     """
     if seed is not None:
         np.random.seed(seed=seed)
@@ -218,7 +217,7 @@ def _rand_herm_dense(N, density, pos_def):
 
 
 def rand_unitary(N, density=0.75, dims=None, seed=None):
-    """Creates a random NxN sparse unitary quantum object.
+    r"""Creates a random NxN sparse unitary quantum object.
 
     Uses :math:`\exp(-iH)` where H is a randomly generated
     Hermitian operator.
@@ -375,7 +374,7 @@ def rand_ket_haar(N=2, dims=None, seed=None):
 
 
 def rand_dm(N, density=0.75, pure=False, dims=None, seed=None):
-    """Creates a random NxN density matrix.
+    r"""Creates a random NxN density matrix.
 
     Parameters
     ----------
@@ -396,7 +395,7 @@ def rand_dm(N, density=0.75, pure=False, dims=None, seed=None):
     Notes
     -----
     For small density matrices., choosing a low density will result in an error
-    as no diagonal elements will be generated such that :math:`Tr(\\rho)=1`.
+    as no diagonal elements will be generated such that :math:`Tr(\rho)=1`.
 
     """
     if isinstance(N, (np.ndarray, list)):
@@ -529,8 +528,7 @@ def rand_kraus_map(N, dims=None, seed=None):
         _check_dims(dims, N, N)
 
     # Random unitary (Stinespring Dilation)
-    big_unitary = rand_unitary(N ** 3, seed=seed).data.todense()
-    orthog_cols = np.array(big_unitary[:, :N])
+    orthog_cols = rand_unitary(N ** 3, seed=seed).full()[:, :N]
     oper_list = np.reshape(orthog_cols, (N ** 2, N, N))
     return list(map(lambda x: Qobj(inpt=x, dims=dims), oper_list))
 
