@@ -50,7 +50,7 @@ from qutip.cy.sparse_utils import (_sparse_profile, _sparse_permute,
                                    _sparse_reverse_permute, _sparse_bandwidth,
                                    _isdiag, zcsr_one_norm, zcsr_inf_norm)
 from qutip.fastsparse import fast_csr_matrix
-from qutip.cy.spconvert import (arr_coo2fast, zcsr_reshape)
+from qutip.cy.spconvert import zcsr_reshape
 from qutip.settings import debug, eigh_unsafe
 
 if eigh_unsafe:
@@ -406,7 +406,7 @@ def sp_eigs(data, isherm, vecs=True, sparse=False, sort='low',
         evals, evecs = _sp_eigs(data, isherm, vecs, N, eigvals, num_large,
                                 num_small, tol, maxiter)
     else:
-        evals, evecs = _dense_eigs(data.todense(), isherm, vecs, N, eigvals,
+        evals, evecs = _dense_eigs(data.toarray(), isherm, vecs, N, eigvals,
                                    num_large, num_small)
 
     if sort == 'high':  # flip arrays to largest values first
