@@ -100,7 +100,7 @@ def basis(dimensions, n=None, offset=None):
 
     Examples
     --------
-    >>> basis(5,2)
+    >>> basis(5,2) # doctest: +SKIP
     Quantum object: dims = [[5], [1]], shape = (5, 1), type = ket
     Qobj data =
     [[ 0.+0.j]
@@ -108,7 +108,7 @@ def basis(dimensions, n=None, offset=None):
      [ 1.+0.j]
      [ 0.+0.j]
      [ 0.+0.j]]
-    >>> basis([2,2,2], [0,1,0])
+    >>> basis([2,2,2], [0,1,0]) # doctest: +SKIP
     Quantum object: dims = [[2, 2, 2], [1, 1, 1]], shape = (8, 1), type = ket
     Qobj data =
     [[0.]
@@ -196,7 +196,7 @@ def coherent(N, alpha, offset=0, method='operator'):
 
     Examples
     --------
-    >>> coherent(5,0.25j)
+    >>> coherent(5,0.25j) # doctest: +SKIP
     Quantum object: dims = [[5], [1]], shape = [5, 1], type = ket
     Qobj data =
     [[  9.69233235e-01+0.j        ]
@@ -269,7 +269,7 @@ def coherent_dm(N, alpha, offset=0, method='operator'):
 
     Examples
     --------
-    >>> coherent_dm(3,0.25j)
+    >>> coherent_dm(3,0.25j) # doctest: +SKIP
     Quantum object: dims = [[3], [3]], \
 shape = [3, 3], type = oper, isHerm = True
     Qobj data =
@@ -332,7 +332,7 @@ def fock_dm(dimensions, n=None, offset=None):
 
     Examples
     --------
-     >>> fock_dm(3,1)
+     >>> fock_dm(3,1) # doctest: +SKIP
      Quantum object: dims = [[3], [3]], \
 shape = [3, 3], type = oper, isHerm = True
      Qobj data =
@@ -374,7 +374,7 @@ def fock(dimensions, n=None, offset=None):
 
     Examples
     --------
-    >>> fock(4,3)
+    >>> fock(4,3) # doctest: +SKIP
     Quantum object: dims = [[4], [1]], shape = [4, 1], type = ket
     Qobj data =
     [[ 0.+0.j]
@@ -408,7 +408,7 @@ def thermal_dm(N, n, method='operator'):
 
     Examples
     --------
-    >>> thermal_dm(5, 1)
+    >>> thermal_dm(5, 1) # doctest: +SKIP
     Quantum object: dims = [[5], [5]], \
 shape = [5, 5], type = oper, isHerm = True
     Qobj data =
@@ -419,7 +419,7 @@ shape = [5, 5], type = oper, isHerm = True
      [ 0.          0.          0.          0.          0.03225806]]
 
 
-    >>> thermal_dm(5, 1, 'analytic')
+    >>> thermal_dm(5, 1, 'analytic') # doctest: +SKIP
     Quantum object: dims = [[5], [5]], \
 shape = [5, 5], type = oper, isHerm = True
     Qobj data =
@@ -499,7 +499,7 @@ def ket2dm(Q):
     Examples
     --------
     >>> x=basis(3,2)
-    >>> ket2dm(x)
+    >>> ket2dm(x) # doctest: +SKIP
     Quantum object: dims = [[3], [3]], \
 shape = [3, 3], type = oper, isHerm = True
     Qobj data =
@@ -521,7 +521,9 @@ shape = [3, 3], type = oper, isHerm = True
 # projection operator
 #
 def projection(N, n, m, offset=None):
-    """The projection operator that projects state :math:`|m>` on state :math:`|n>`.
+    r"""
+    The projection operator that projects state :math:`\lvert m\rangle` on
+    state :math:`\lvert n\rangle`.
 
     Parameters
     ----------
@@ -539,7 +541,6 @@ def projection(N, n, m, offset=None):
     -------
     oper : qobj
          Requested projection operator.
-
     """
     ket1 = basis(N, n, offset=offset)
     ket2 = basis(N, m, offset=offset)
@@ -551,8 +552,8 @@ def projection(N, n, m, offset=None):
 # composite qubit states
 #
 def qstate(string):
-    """Creates a tensor product for a set of qubits in either
-    the 'up' :math:`|0>` or 'down' :math:`|1>` state.
+    r"""Creates a tensor product for a set of qubits in either
+    the 'up' :math:`\lvert0\rangle` or 'down' :math:`\lvert1\rangle` state.
 
     Parameters
     ----------
@@ -571,7 +572,7 @@ def qstate(string):
 
     Examples
     --------
-    >>> qstate('udu')
+    >>> qstate('udu') # doctest: +SKIP
     Quantum object: dims = [[2, 2, 2], [1, 1, 1]], shape = [8, 1], type = ket
     Qobj data =
     [[ 0.]
@@ -582,7 +583,6 @@ def qstate(string):
      [ 1.]
      [ 0.]
      [ 0.]]
-
     """
     n = len(string)
     if n != (string.count('u') + string.count('d')):
@@ -648,7 +648,7 @@ def ket(seq, dim=2):
 
     Examples
     --------
-    >>> ket("10")
+    >>> ket("10") # doctest: +SKIP
     Quantum object: dims = [[2, 2], [1, 1]], shape = [4, 1], type = ket
     Qobj data =
     [[ 0.]
@@ -656,7 +656,7 @@ def ket(seq, dim=2):
      [ 1.]
      [ 0.]]
 
-    >>> ket("Hue")
+    >>> ket("Hue") # doctest: +SKIP
     Quantum object: dims = [[2, 2, 2], [1, 1, 1]], shape = [8, 1], type = ket
     Qobj data =
     [[ 0.]
@@ -668,7 +668,7 @@ def ket(seq, dim=2):
      [ 0.]
      [ 0.]]
 
-    >>> ket("12", 3)
+    >>> ket("12", 3) # doctest: +SKIP
     Quantum object: dims = [[3, 3], [1, 1]], shape = [9, 1], type = ket
     Qobj data =
     [[ 0.]
@@ -681,7 +681,7 @@ def ket(seq, dim=2):
      [ 0.]
      [ 0.]]
 
-    >>> ket("31", [5, 2])
+    >>> ket("31", [5, 2]) # doctest: +SKIP
     Quantum object: dims = [[5, 2], [1, 1]], shape = [10, 1], type = ket
     Qobj data =
     [[ 0.]
@@ -712,9 +712,11 @@ def bra(seq, dim=2):
         Each element defines state of the respective particle.
         (e.g. [1,1,0,1] or a string "1101").
         For qubits it is also possible to use the following conventions:
+
         - 'g'/'e' (ground and excited state)
         - 'u'/'d' (spin up and down)
         - 'H'/'V' (horizontal and vertical polarization)
+
         Note: for dimension > 9 you need to use a list.
 
 
@@ -728,23 +730,23 @@ def bra(seq, dim=2):
 
     Examples
     --------
-    >>> bra("10")
+    >>> bra("10") # doctest: +SKIP
     Quantum object: dims = [[1, 1], [2, 2]], shape = [1, 4], type = bra
     Qobj data =
     [[ 0.  0.  1.  0.]]
 
-    >>> bra("Hue")
+    >>> bra("Hue") # doctest: +SKIP
     Quantum object: dims = [[1, 1, 1], [2, 2, 2]], shape = [1, 8], type = bra
     Qobj data =
     [[ 0.  1.  0.  0.  0.  0.  0.  0.]]
 
-    >>> bra("12", 3)
+    >>> bra("12", 3) # doctest: +SKIP
     Quantum object: dims = [[1, 1], [3, 3]], shape = [1, 9], type = bra
     Qobj data =
     [[ 0.  0.  0.  0.  0.  1.  0.  0.  0.]]
 
 
-    >>> bra("31", [5, 2])
+    >>> bra("31", [5, 2]) # doctest: +SKIP
     Quantum object: dims = [[1, 1], [5, 2]], shape = [1, 10], type = bra
     Qobj data =
     [[ 0.  0.  0.  0.  0.  0.  0.  1.  0.  0.]]
@@ -762,8 +764,8 @@ def state_number_enumerate(dims, excitations=None, state=None, idx=0):
 
     Example:
 
-        >>> for state in state_number_enumerate([2,2]):
-        >>>     print(state)
+        >>> for state in state_number_enumerate([2,2]): # doctest: +SKIP
+        >>>     print(state) # doctest: +SKIP
         [ 0  0 ]
         [ 0  1 ]
         [ 1  0 ]
@@ -881,7 +883,7 @@ def state_number_qobj(dims, state):
 
     Example:
 
-        >>> state_number_qobj([2, 2, 2], [1, 0, 1])
+        >>> state_number_qobj([2, 2, 2], [1, 0, 1]) # doctest: +SKIP
         Quantum object: dims = [[2, 2, 2], [1, 1, 1]], \
 shape = [8, 1], type = ket
         Qobj data =
@@ -1088,8 +1090,8 @@ def zero_ket(N, dims=None):
 
 
 def spin_state(j, m, type='ket'):
-    """Generates the spin state |j, m>, i.e.  the eigenstate
-    of the spin-j Sz operator with eigenvalue m.
+    r"""Generates the spin state :math:`\lvert j, m\rangle`, i.e. the
+    eigenstate of the spin-j Sz operator with eigenvalue m.
 
     Parameters
     ----------
@@ -1106,7 +1108,6 @@ def spin_state(j, m, type='ket'):
     -------
     state : qobj
         Qobj quantum object for spin state
-
     """
     J = 2 * j + 1
 
@@ -1121,7 +1122,7 @@ def spin_state(j, m, type='ket'):
 
 
 def spin_coherent(j, theta, phi, type='ket'):
-    """Generate the coherent spin state |theta, phi>.
+    r"""Generate the coherent spin state :math:`\lvert \theta, \phi\rangle`.
 
     Parameters
     ----------
@@ -1141,7 +1142,6 @@ def spin_coherent(j, theta, phi, type='ket'):
     -------
     state : qobj
         Qobj quantum object for spin coherent state
-
     """
     Sp = jmat(j, '+')
     Sm = jmat(j, '-')
@@ -1159,19 +1159,26 @@ def spin_coherent(j, theta, phi, type='ket'):
 
 
 def bell_state(state='00'):
-    """
-    Returns the Bell state:
+    r"""
+    Returns the selected Bell state:
 
-        |B00> = 1 / sqrt(2)*[|0>|0>+|1>|1>]
-        |B01> = 1 / sqrt(2)*[|0>|0>-|1>|1>]
-        |B10> = 1 / sqrt(2)*[|0>|1>+|1>|0>]
-        |B11> = 1 / sqrt(2)*[|0>|1>-|1>|0>]
+    .. math::
+
+        \begin{aligned}
+        \lvert B_{00}\rangle &=
+            \frac1{\sqrt2}(\lvert00\rangle+\lvert11\rangle)\\
+        \lvert B_{01}\rangle &=
+            \frac1{\sqrt2}(\lvert00\rangle-\lvert11\rangle)\\
+        \lvert B_{10}\rangle &=
+            \frac1{\sqrt2}(\lvert01\rangle+\lvert10\rangle)\\
+        \lvert B_{11}\rangle &=
+            \frac1{\sqrt2}(\lvert01\rangle-\lvert10\rangle)\\
+        \end{aligned}
 
     Returns
     -------
     Bell_state : qobj
         Bell state
-
     """
     if state == '00':
         Bell_state = tensor(
@@ -1190,30 +1197,32 @@ def bell_state(state='00'):
 
 
 def singlet_state():
-    """
+    r"""
     Returns the two particle singlet-state:
 
-        |S>=1/sqrt(2)*[|0>|1>-|1>|0>]
+    .. math::
+
+        \lvert S\rangle = \frac1{\sqrt2}(\lvert01\rangle-\lvert10\rangle)
 
     that is identical to the fourth bell state.
 
     Returns
     -------
     Bell_state : qobj
-        |B11> Bell state
-
+        :math:`\lvert B_{11}\rangle` Bell state
     """
     return bell_state('11')
 
 
 def triplet_states():
-    """
-    Returns the two particle triplet-states:
+    r"""
+    Returns a list of the two particle triplet-states:
 
-        |T>= |1>|1>
-           = 1 / sqrt(2)*[|0>|1>-|1>|0>]
-           = |0>|0>
-    that is identical to the fourth bell state.
+    .. math::
+
+        \lvert T_1\rangle = \lvert11\rangle
+        \lvert T_2\rangle = \frac1{\sqrt2}(\lvert01\rangle - \lvert10\rangle)
+        \lvert T_3\rangle = \lvert00\rangle
 
     Returns
     -------
