@@ -39,29 +39,27 @@ collapse operators.
 __all__ = ['steadystate', 'steady', 'build_preconditioner',
            'pseudo_inverse']
 
-import warnings
+import functools
 import time
-import scipy
+import warnings
+
+from packaging.version import parse as _parse_version
 import numpy as np
 from numpy.linalg import svd
-from scipy import prod
+import scipy
 import scipy.sparse as sp
 import scipy.linalg as la
-from scipy.sparse.linalg import (use_solver, splu, spilu, spsolve, eigs,
-                                 LinearOperator, gmres, lgmres, bicgstab)
+from scipy.sparse.linalg import (
+    use_solver, splu, spilu, eigs, LinearOperator, gmres, lgmres, bicgstab,
+)
 from qutip.qobj import Qobj, issuper, isoper
 
 from qutip.superoperator import liouvillian, vec2mat, spre
-from qutip.sparse import sp_permute, sp_bandwidth, sp_reshape, sp_profile
-
-from qutip.superoperator import liouvillian, vec2mat
-from qutip.sparse import (sp_permute, sp_bandwidth, sp_reshape,
-                          sp_profile)
+from qutip.sparse import sp_permute, sp_bandwidth, sp_profile
 from qutip.cy.spmath import zcsr_kron
 from qutip.graph import weighted_bipartite_matching
 from qutip import (mat2vec, tensor, identity, operator_to_vector)
 import qutip.settings as settings
-from qutip.utilities import _version2int
 from qutip.cy.spconvert import dense2D_to_fastcsr_fmode
 
 import qutip.logging_utils
@@ -72,8 +70,35 @@ logger.setLevel('DEBUG')
 if settings.has_mkl:
     from qutip._mkl.spsolve import (mkl_splu, mkl_spsolve)
 
-# test if scipy is recent enought to get L & U factors from superLU
-_scipy_check = _version2int(scipy.__version__) >= _version2int('0.14.0')
+
+def _eat_kwargs(function, names):
+    """
+    Return a wrapped version of `function` that simply removes any keyword
+    arguments with one of the given names.
+    """
+    @functools.wraps(function)
+    def out(*args, **kwargs):
+        for name in names:
+            if name in kwargs:
+                del kwargs[name]
+        return function(*args, **kwargs)
+    return out
+
+
+# From SciPy 1.4 onwards we need to pass the `callback_type='legacy'` argument
+# to gmres to maintain the same behaviour we used to have.  Since this should
+# be the default behaviour, we use that in the main code and just "eat" the
+# argument if passed to a lower version of SciPy that doesn't know about it.
+# Similarly, SciPy < 1.1 does not recognise the `atol` keyword.
+#
+# Respective checks can be removed when SciPy version requirements are raised.
+
+if _parse_version(scipy.__version__) < _parse_version("1.1"):
+    gmres = _eat_kwargs(gmres, ['atol', 'callback_type'])
+    lgmres = _eat_kwargs(lgmres, ['atol'])
+    bicgstab = _eat_kwargs(bicgstab, ['atol'])
+elif _parse_version(scipy.__version__) < _parse_version("1.4"):
+    gmres = _eat_kwargs(gmres, ['callback_type'])
 
 
 def _empty_info_dict():
@@ -101,50 +126,62 @@ def _default_steadystate_args():
 
 
 def steadystate(A, c_op_list=[], method='direct', solver=None, **kwargs):
-    """Calculates the steady state for quantum evolution subject to the
-    supplied Hamiltonian or Liouvillian operator and (if given a Hamiltonian) a
-    list of collapse operators.
+    """
+    Calculates the steady state for quantum evolution subject to the supplied
+    Hamiltonian or Liouvillian operator and (if given a Hamiltonian) a list of
+    collapse operators.
 
     If the user passes a Hamiltonian then it, along with the list of collapse
     operators, will be converted into a Liouvillian operator in Lindblad form.
 
     Parameters
     ----------
-    A : qobj
+    A : :obj:`~Qobj`
         A Hamiltonian or Liouvillian operator.
 
     c_op_list : list
         A list of collapse operators.
 
-    solver : str {None, 'scipy', 'mkl'}
-        Selects the sparse solver to use.  Default is auto-select
-        based on the availability of the MKL library.
+    solver : {'scipy', 'mkl'}, optional
+        Selects the sparse solver to use.  Default is to auto-select based on
+        the availability of the MKL library.
+
+    method : str, default 'direct'
+        The allowed methods are
+
+        - 'direct'
+        - 'eigen'
+        - 'iterative-gmres'
+        - 'iterative-lgmres'
+        - 'iterative-bicgstab'
+        - 'svd'
+        - 'power'
+        - 'power-gmres'
+        - 'power-lgmres'
+        - 'power-bicgstab'
 
-    method : str {'direct', 'eigen', 'iterative-gmres',
-                  'iterative-lgmres', 'iterative-bicgstab', 'svd', 'power',
-                  'power-gmres', 'power-lgmres', 'power-bicgstab'}
         Method for solving the underlying linear equation. Direct LU solver
-        'direct' (default), sparse eigenvalue problem 'eigen',
-        iterative GMRES method 'iterative-gmres', iterative LGMRES method
-        'iterative-lgmres', iterative BICGSTAB method 'iterative-bicgstab',
-        SVD 'svd' (dense), or inverse-power method 'power'. The iterative
-        power methods 'power-gmres', 'power-lgmres', 'power-bicgstab' use
-        the same solvers as their direct counterparts.
+        'direct' (default), sparse eigenvalue problem 'eigen', iterative GMRES
+        method 'iterative-gmres', iterative LGMRES method 'iterative-lgmres',
+        iterative BICGSTAB method 'iterative-bicgstab', SVD 'svd' (dense), or
+        inverse-power method 'power'. The iterative power methods
+        'power-gmres', 'power-lgmres', 'power-bicgstab' use the same solvers as
+        their direct counterparts.
 
-    return_info : bool, optional, default = False
-        Return a dictionary of solver-specific infomation about the
-        solution and how it was obtained.
+    return_info : bool, default False
+        Return a dictionary of solver-specific infomation about the solution
+        and how it was obtained.
 
-    sparse : bool, optional, default = True
+    sparse : bool, default True
         Solve for the steady state using sparse algorithms. If set to False,
         the underlying Liouvillian operator will be converted into a dense
         matrix. Use only for 'smaller' systems.
 
-    use_rcm : bool, optional, default = False
-        Use reverse Cuthill-Mckee reordering to minimize fill-in in the
-        LU factorization of the Liouvillian.
+    use_rcm : bool, default False
+        Use reverse Cuthill-Mckee reordering to minimize fill-in in the LU
+        factorization of the Liouvillian.
 
-    use_wbm : bool, optional, default = False
+    use_wbm : bool, default False
         Use Weighted Bipartite Matching reordering to make the Liouvillian
         diagonally dominant.  This is useful for iterative preconditioners
         only, and is set to ``True`` by default when finding a preconditioner.
@@ -154,41 +191,41 @@ def steadystate(A, c_op_list=[], method='direct', solver=None, **kwargs):
         to the linear solvers.  This is set to the average abs value of the
         Liouvillian elements if not specified by the user.
 
-    max_iter_refine : int {10}
+    max_iter_refine : int, default 10
         MKL ONLY. Max. number of iterative refinements to perform.
 
-    scaling_vectors : bool {True, False}
+    scaling_vectors : bool
         MKL ONLY.  Scale matrix to unit norm columns and rows.
 
-    weighted_matching : bool {True, False}
+    weighted_matching : bool
         MKL ONLY.  Use weighted matching to better condition diagonal.
 
     x0 : ndarray, optional
         ITERATIVE ONLY. Initial guess for solution vector.
 
-    maxiter : int, optional, default=1000
+    maxiter : int, default 1000
         ITERATIVE ONLY. Maximum number of iterations to perform.
 
-    tol : float, optional, default=1e-12
+    tol : float, default 1e-12
         ITERATIVE ONLY. Tolerance used for terminating solver.
 
-    mtol : float, optional, default=None
-        ITERATIVE 'power' methods ONLY. Tolerance for lu solve method.
-        If None given then `max(0.1*tol, 1e-15)` is used
+    mtol : float, optional
+        ITERATIVE 'power' methods ONLY. Tolerance for lu solve method.  If None
+        given then ``max(0.1*tol, 1e-15)`` is used.
 
-    matol : float, optional, default=1e-15
+    matol : float, default 1e-15
         ITERATIVE ONLY. Absolute tolerance for lu solve method.
 
-    permc_spec : str, optional, default='COLAMD'
+    permc_spec : str, optional
         ITERATIVE ONLY. Column ordering used internally by superLU for the
-        'direct' LU decomposition method. Options include 'COLAMD' and
-        'NATURAL'. If using RCM then this is set to 'NATURAL' automatically
+        'direct' LU decomposition method. Options include 'COLAMD' (default)
+        and 'NATURAL'. If using RCM then this is set to 'NATURAL' automatically
         unless explicitly specified.
 
-    use_precond : bool optional, default = False
+    use_precond : bool, default False
         ITERATIVE ONLY. Use an incomplete sparse LU decomposition as a
-        preconditioner for the 'iterative' GMRES and BICG solvers.
-        Speeds up convergence time by orders of magnitude in many cases.
+        preconditioner for the 'iterative' GMRES and BICG solvers.  Speeds up
+        convergence time by orders of magnitude in many cases.
 
     M : {sparse matrix, dense matrix, LinearOperator}, optional
         ITERATIVE ONLY. Preconditioner for A. The preconditioner should
@@ -197,27 +234,27 @@ def steadystate(A, c_op_list=[], method='direct', solver=None, **kwargs):
         If no preconditioner is given and ``use_precond = True``, then one
         is generated automatically.
 
-    fill_factor : float, optional, default = 100
+    fill_factor : float, default 100
         ITERATIVE ONLY. Specifies the fill ratio upper bound (>=1) of the iLU
         preconditioner.  Lower values save memory at the cost of longer
         execution times and a possible singular factorization.
 
-    drop_tol : float, optional, default = 1e-4
+    drop_tol : float, default 1e-4
         ITERATIVE ONLY. Sets the threshold for the magnitude of preconditioner
         elements that should be dropped.  Can be reduced for a courser
         factorization at the cost of an increased number of iterations, and a
         possible singular factorization.
 
-    diag_pivot_thresh : float, optional, default = None
+    diag_pivot_thresh : float, optional
         ITERATIVE ONLY. Sets the threshold between [0,1] for which diagonal
         elements are considered acceptable pivot points when using a
         preconditioner.  A value of zero forces the pivot to be the diagonal
         element.
 
-    ILU_MILU : str, optional, default = 'smilu_2'
-        ITERATIVE ONLY. Selects the incomplete LU decomposition method
-        algoithm used in creating the preconditoner. Should only be used by
-        advanced users.
+    ILU_MILU : str, default 'smilu_2'
+        ITERATIVE ONLY. Selects the incomplete LU decomposition method algoithm
+        used in creating the preconditoner. Should only be used by advanced
+        users.
 
     Returns
     -------
@@ -229,7 +266,6 @@ def steadystate(A, c_op_list=[], method='direct', solver=None, **kwargs):
     Notes
     -----
     The SVD method works only for dense operators (i.e. small systems).
-
     """
     if solver is None:
         solver = 'scipy'
@@ -423,7 +459,7 @@ def _steadystate_direct_sparse(L, ss_args):
         v = lu.solve(b)
         _direct_end = time.time()
         ss_args['info']['solution_time'] = _direct_end - _direct_start
-        if (settings.debug or ss_args['return_info']) and _scipy_check:
+        if (settings.debug or ss_args['return_info']):
             L_nnz = lu.L.nnz
             U_nnz = lu.U.nnz
             ss_args['info']['l_nnz'] = L_nnz
@@ -465,8 +501,8 @@ def _steadystate_direct_sparse(L, ss_args):
 
 def _steadystate_direct_dense(L, ss_args):
     """
-    Direct solver that use numpy dense matrices. Suitable for
-    small system, with a few states.
+    Direct solver that uses numpy arrays. Suitable for small systems with few
+    states.
     """
     if settings.debug:
         logger.debug('Starting direct dense solver.')
@@ -476,14 +512,14 @@ def _steadystate_direct_dense(L, ss_args):
     b = np.zeros(n ** 2)
     b[0] = ss_args['weight']
 
-    L = L.data.todense()
+    L = L.full()
     L[0, :] = np.diag(ss_args['weight']*np.ones(n)).reshape((1, n ** 2))
     _dense_start = time.time()
     v = np.linalg.solve(L, b)
     _dense_end = time.time()
     ss_args['info']['solution_time'] = _dense_end-_dense_start
     if ss_args['return_info']:
-        ss_args['info']['residual_norm'] = la.norm(b - L*v, np.inf)
+        ss_args['info']['residual_norm'] = la.norm(b - L@v, np.inf)
     data = vec2mat(v)
     data = 0.5 * (data + data.conj().T)
 
@@ -566,20 +602,18 @@ def _iterative_precondition(A, n, ss_args):
                 logger.debug('Preconditioning succeeded.')
                 logger.debug('Precond. time: %f' %
                              (_precond_end - _precond_start))
-
-            if _scipy_check:
-                L_nnz = P.L.nnz
-                U_nnz = P.U.nnz
-                ss_args['info']['l_nnz'] = L_nnz
-                ss_args['info']['u_nnz'] = U_nnz
-                ss_args['info']['ilu_fill_factor'] = (L_nnz+U_nnz)/A.nnz
-                e = np.ones(n ** 2, dtype=int)
-                condest = la.norm(M*e, np.inf)
-                ss_args['info']['ilu_condest'] = condest
-                if settings.debug:
-                    logger.debug('L NNZ: %i ; U NNZ: %i' % (L_nnz, U_nnz))
-                    logger.debug('Fill factor: %f' % ((L_nnz+U_nnz)/A.nnz))
-                    logger.debug('iLU condest: %f' % condest)
+            L_nnz = P.L.nnz
+            U_nnz = P.U.nnz
+            ss_args['info']['l_nnz'] = L_nnz
+            ss_args['info']['u_nnz'] = U_nnz
+            ss_args['info']['ilu_fill_factor'] = (L_nnz+U_nnz)/A.nnz
+            e = np.ones(n ** 2, dtype=int)
+            condest = la.norm(M*e, np.inf)
+            ss_args['info']['ilu_condest'] = condest
+            if settings.debug:
+                logger.debug('L NNZ: %i ; U NNZ: %i' % (L_nnz, U_nnz))
+                logger.debug('Fill factor: %f' % ((L_nnz+U_nnz)/A.nnz))
+                logger.debug('iLU condest: %f' % condest)
 
     except:
         raise Exception("Failed to build preconditioner. Try increasing " +
@@ -623,50 +657,23 @@ def _steadystate_iterative(L, ss_args):
 
     # Select iterative solver type
     _iter_start = time.time()
-    # FIXME: These atol keyword except checks can be removed once scipy 1.1
-    # is a minimum requirement
-    extra = {"callback_type": 'legacy'} if scipy.__version__ >= "1.4" else {}
     if ss_args['method'] == 'iterative-gmres':
-        try:
-            v, check = gmres(L, b, tol=ss_args['tol'], atol=ss_args['matol'],
-                             M=ss_args['M'], x0=ss_args['x0'],
-                             restart=ss_args['restart'],
-                             maxiter=ss_args['maxiter'],
-                             callback=_iter_count, **extra)
-        except TypeError as e:
-            if "unexpected keyword argument 'atol'" in str(e):
-                v, check = gmres(L, b, tol=ss_args['tol'],
-                                 M=ss_args['M'], x0=ss_args['x0'],
-                                 restart=ss_args['restart'],
-                                 maxiter=ss_args['maxiter'],
-                                 callback=_iter_count)
-
+        v, check = gmres(L, b, tol=ss_args['tol'], atol=ss_args['matol'],
+                         M=ss_args['M'], x0=ss_args['x0'],
+                         restart=ss_args['restart'],
+                         maxiter=ss_args['maxiter'],
+                         callback=_iter_count, callback_type='legacy')
     elif ss_args['method'] == 'iterative-lgmres':
-        try:
-            v, check = lgmres(L, b, tol=ss_args['tol'], atol=ss_args['matol'],
-                              M=ss_args['M'], x0=ss_args['x0'],
-                              maxiter=ss_args['maxiter'],
-                              callback=_iter_count)
-        except TypeError as e:
-            if "unexpected keyword argument 'atol'" in str(e):
-                v, check = lgmres(L, b, tol=ss_args['tol'],
-                                  M=ss_args['M'], x0=ss_args['x0'],
-                                  maxiter=ss_args['maxiter'],
-                                  callback=_iter_count)
-
+        v, check = lgmres(L, b, tol=ss_args['tol'], atol=ss_args['matol'],
+                          M=ss_args['M'], x0=ss_args['x0'],
+                          maxiter=ss_args['maxiter'],
+                          callback=_iter_count)
     elif ss_args['method'] == 'iterative-bicgstab':
-        try:
-            v, check = bicgstab(L, b, tol=ss_args['tol'],
-                                atol=ss_args['matol'],
-                                M=ss_args['M'], x0=ss_args['x0'],
-                                maxiter=ss_args['maxiter'],
-                                callback=_iter_count)
-        except TypeError as e:
-            if "unexpected keyword argument 'atol'" in str(e):
-                v, check = bicgstab(L, b, tol=ss_args['tol'],
-                                    M=ss_args['M'], x0=ss_args['x0'],
-                                    maxiter=ss_args['maxiter'],
-                                    callback=_iter_count)
+        v, check = bicgstab(L, b, tol=ss_args['tol'],
+                            atol=ss_args['matol'],
+                            M=ss_args['M'], x0=ss_args['x0'],
+                            maxiter=ss_args['maxiter'],
+                            callback=_iter_count)
     else:
         raise Exception("Invalid iterative solver method.")
     _iter_end = time.time()
@@ -863,59 +870,33 @@ def _steadystate_power(L, ss_args):
                       diag_pivot_thresh=ss_args['diag_pivot_thresh'],
                       options=dict(ILU_MILU=ss_args['ILU_MILU']))
 
-            if settings.debug and _scipy_check:
+            if settings.debug:
                 L_nnz = lu.L.nnz
                 U_nnz = lu.U.nnz
                 logger.debug('L NNZ: %i ; U NNZ: %i' % (L_nnz, U_nnz))
                 logger.debug('Fill factor: %f' % ((L_nnz+U_nnz)/orig_nnz))
 
     it = 0
-    # FIXME: These atol keyword except checks can be removed once scipy 1.1
-    # is a minimum requirement
     while (la.norm(L * v, np.inf) > tol) and (it < maxiter):
         check = 0
         if ss_args['method'] == 'power':
             v = lu.solve(v)
         elif ss_args['method'] == 'power-gmres':
-            try:
-                v, check = gmres(L, v, tol=mtol, atol=ss_args['matol'],
-                                 M=ss_args['M'], x0=ss_args['x0'],
-                                 restart=ss_args['restart'],
-                                 maxiter=ss_args['maxiter'],
-                                 callback=_iter_count)
-            except TypeError as e:
-                if "unexpected keyword argument 'atol'" in str(e):
-                    v, check = gmres(L, v, tol=mtol,
-                                     M=ss_args['M'], x0=ss_args['x0'],
-                                     restart=ss_args['restart'],
-                                     maxiter=ss_args['maxiter'],
-                                     callback=_iter_count)
-
+            v, check = gmres(L, v, tol=mtol, atol=ss_args['matol'],
+                             M=ss_args['M'], x0=ss_args['x0'],
+                             restart=ss_args['restart'],
+                             maxiter=ss_args['maxiter'],
+                             callback=_iter_count, callback_type='legacy')
         elif ss_args['method'] == 'power-lgmres':
-            try:
-                v, check = lgmres(L, v, tol=mtol, atol=ss_args['matol'],
-                                  M=ss_args['M'], x0=ss_args['x0'],
-                                  maxiter=ss_args['maxiter'],
-                                  callback=_iter_count)
-            except TypeError as e:
-                if "unexpected keyword argument 'atol'" in str(e):
-                    v, check = lgmres(L, v, tol=mtol,
-                                      M=ss_args['M'], x0=ss_args['x0'],
-                                      maxiter=ss_args['maxiter'],
-                                      callback=_iter_count)
-
+            v, check = lgmres(L, v, tol=mtol, atol=ss_args['matol'],
+                              M=ss_args['M'], x0=ss_args['x0'],
+                              maxiter=ss_args['maxiter'],
+                              callback=_iter_count)
         elif ss_args['method'] == 'power-bicgstab':
-            try:
-                v, check = bicgstab(L, v, tol=mtol, atol=ss_args['matol'],
-                                    M=ss_args['M'], x0=ss_args['x0'],
-                                    maxiter=ss_args['maxiter'],
-                                    callback=_iter_count)
-            except TypeError as e:
-                if "unexpected keyword argument 'atol'" in str(e):
-                    v, check = bicgstab(L, v, tol=mtol,
-                                        M=ss_args['M'], x0=ss_args['x0'],
-                                        maxiter=ss_args['maxiter'],
-                                        callback=_iter_count)
+            v, check = bicgstab(L, v, tol=mtol, atol=ss_args['matol'],
+                                M=ss_args['M'], x0=ss_args['x0'],
+                                maxiter=ss_args['maxiter'],
+                                callback=_iter_count)
         else:
             raise Exception("Invalid iterative solver method.")
         if check > 0:
