@@ -35,10 +35,10 @@ __all__ = ['wigner', 'qfunc', 'spin_q_function',
            'spin_wigner', 'wigner_transform']
 
 import numpy as np
-from numpy import (zeros, array, arange, exp, real, conj, pi,
-                   copy, sqrt, meshgrid, size, conjugate,
-                   cos, sin)
-from scipy import polyval, fliplr
+from numpy import (
+    zeros, array, arange, exp, real, conj, pi, copy, sqrt, meshgrid, size,
+    conjugate, cos, sin, polyval, fliplr,
+)
 import scipy.sparse as sp
 import scipy.fftpack as ft
 import scipy.linalg as la
@@ -291,12 +291,12 @@ def wigner(psi, xvec, yvec, method='clenshaw', g=sqrt(2),
 
 
 def _wigner_iterative(rho, xvec, yvec, g=sqrt(2)):
-    """
+    r"""
     Using an iterative method to evaluate the wigner functions for the Fock
     state :math:`|m><n|`.
 
     The Wigner function is calculated as
-    :math:`W = \sum_{mn} \\rho_{mn} W_{mn}` where :math:`W_{mn}` is the Wigner
+    :math:`W = \sum_{mn} \rho_{mn} W_{mn}` where :math:`W_{mn}` is the Wigner
     function for the density matrix :math:`|m><n|`.
 
     In this implementation, for each row m, Wlist contains the Wigner functions
@@ -337,10 +337,10 @@ def _wigner_iterative(rho, xvec, yvec, g=sqrt(2)):
 
 
 def _wigner_laguerre(rho, xvec, yvec, g, parallel):
-    """
+    r"""
     Using Laguerre polynomials from scipy to evaluate the Wigner function for
     the density matrices :math:`|m><n|`, :math:`W_{mn}`. The total Wigner
-    function is calculated as :math:`W = \sum_{mn} \\rho_{mn} W_{mn}`.
+    function is calculated as :math:`W = \sum_{mn} \rho_{mn} W_{mn}`.
     """
 
     M = np.prod(rho.shape[0])
@@ -479,14 +479,14 @@ def _osc_eigen(N, pnts):
 
 
 def _wigner_clenshaw(rho, xvec, yvec, g=sqrt(2), sparse=False):
-    """
+    r"""
     Using Clenshaw summation - numerically stable and efficient
     iterative algorithm to evaluate polynomial series.
 
     The Wigner function is calculated as
-    :math:`W = e^(-0.5*x^2)/pi * \sum_{L} c_L (2x)^L / sqrt(L!)` where
-    :math:`c_L = \sum_n \\rho_{n,L+n} LL_n^L` where
-    :math:`LL_n^L = (-1)^n sqrt(L!n!/(L+n)!) LaguerreL[n,L,x]`
+    :math:`W = e^(-0.5*x^2)/pi * \sum_{L} c_L (2x)^L / \sqrt(L!)` where
+    :math:`c_L = \sum_n \rho_{n,L+n} LL_n^L` where
+    :math:`LL_n^L = (-1)^n \sqrt(L!n!/(L+n)!) LaguerreL[n,L,x]`
 
     """
 
@@ -499,7 +499,7 @@ def _wigner_clenshaw(rho, xvec, yvec, g=sqrt(2), sparse=False):
     B *= B
     w0 = (2*rho.data[0,-1])*np.ones_like(A2)
     L = M-1
-    #calculation of \sum_{L} c_L (2x)^L / sqrt(L!)
+    #calculation of \sum_{L} c_L (2x)^L / \sqrt(L!)
     #using Horner's method
     if not sparse:
         rho = rho.full() * (2*np.ones((M,M)) - np.diag(np.ones(M)))
@@ -521,12 +521,12 @@ def _wigner_clenshaw(rho, xvec, yvec, g=sqrt(2), sparse=False):
 
 
 def _wig_laguerre_val(L, x, c):
-    """
+    r"""
     this is evaluation of polynomial series inspired by hermval from numpy.
     Returns polynomial series
     \sum_n b_n LL_n^L,
     where
-    LL_n^L = (-1)^n sqrt(L!n!/(L+n)!) LaguerreL[n,L,x]
+    LL_n^L = (-1)^n \sqrt(L!n!/(L+n)!) LaguerreL[n,L,x]
     The evaluation uses Clenshaw recursion
     """
 
